%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Author: Sergey Fomel sergey@sep.stanford.edu
%
% Using some ideas of Herbert W. Swan and Joe Dellinger
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\NeedsTeXFormat{LaTeX2e}
\ProvidesClass{seg-99}[1999/03/16 v0.1 SEG abstract format]
\RequirePackage{ifthen} % for \ifthenelse

\newcommand{\@fonts}{cmfonts} 

% LY1 encoding and MathTime from Y&Y: www.yandy.com
\DeclareOption{yandy}{%
\renewcommand{\@fonts}{yandy}
}

% pslatex from /macros/latex/contrib/supported/pslatex/
\DeclareOption{pslatex}{%
\renewcommand{\@fonts}{pslatex}
}

% times from /macros/latex/packages/psnfss/
\DeclareOption{times}{% 
\renewcommand{\@fonts}{times}
}

\DeclareOption*{\PassOptionsToClass{\CurrentOption}{article}}
\ProcessOptions*

\LoadClass[11pt]{article}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Different options for postscript fonts
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\ifthenelse{\equal{\@fonts}{yandy}}{ 
\RequirePackage[LY1]{fontenc}
\RequirePackage[LY1,mtbold]{mathtime}
\RequirePackage{times}               
}{\ifthenelse{\equal{\@fonts}{pslatex}}{ 
\RequirePackage{pslatex}
}{\ifthenelse{\equal{\@fonts}{times}}{
\RequirePackage{times}
}{}}}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Additional packages
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\RequirePackage{graphicx}     % for \plot 
\RequirePackage{natbib}       % for \cite, \longcite, and \shortcite
\RequirePackage[wavy]{vector} % for \st

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Title commands
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\newcommand{\seg@title}{}
\renewcommand{\title}[1]{\renewcommand{\seg@title}{#1}}

\providecommand{\runninghead}{}
\providecommand{\righthead}[1]{\renewcommand{\runninghead}{#1}}

\newcommand{\seg@author}{}
\renewcommand{\author}[1]{%
\renewcommand{\seg@author}{%
\twocolumn[\vspace{0.25in}\normal@size\textbf{\seg@title}%
\newline\small\textit{#1}\vspace{2ex}]}}

\renewcommand{\maketitle}{\seg@author}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Messing with sizes to get the required 9pt Times-Roman
% A better way is probably to rescale .fd files 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\renewcommand{\Large}{\protect\scriptsize}
\let\normal@size\normalsize
\renewcommand{\normalsize}{\protect\scriptsize}
\renewcommand{\footnotesize}{\protect\tiny}
\scriptsize  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Section commands
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\renewcommand{\section}[2][]{\parbox[t]{\columnwidth}{%
\textbf{\MakeUppercase{#2}}\\*}}

\renewcommand{\subsection}[2][]{%
\begin{minipage}{\columnwidth}
\textbf{#2} 
\end{minipage}}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% abstract is an environment, ABS is a command
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\renewenvironment{abstract}{%
\begin{minipage}[t]{\columnwidth}
\textbf{SUMMARY}\\*}{%
\end{minipage}\vspace{2ex}}

\DeclareRobustCommand{\ABS}[1]{%
\begin{minipage}[t]{\columnwidth}
\textbf{SUMMARY}\\*
#1
\end{minipage}\vspace{2ex}}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Size definitions
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\setlength{\oddsidemargin}{0in}
\setlength{\evensidemargin}{0in}
\setlength{\textwidth}{6.5in}
\setlength{\textheight}{9.0625in}
\setlength{\parindent}{0em}
\setlength{\headheight}{0.3125in}
\setlength{\headsep}{0.125in}
\setlength{\parskip}{1.5ex plus0.1ex minus0.1ex}
\setlength{\topmargin}{-0.5in}
\setlength{\columnsep}{0.3125in}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Page style
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\newcommand{\ps@seg}{%
\renewcommand{\@oddhead}{\hfill\small\textbf{\runninghead}\scriptsize\hfill}
\renewcommand{\@evenhead}{\@oddhead}
\renewcommand{\@oddfoot}{}
\renewcommand{\@evenfoot}{}}

\thispagestyle{empty}
\pagestyle{seg}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Figure directories
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\providecommand{\ext}{ps}
\providecommand{\@pathname}{.}
\providecommand{\inputdir}[1]{\renewcommand{\@pathname}{#1}}

\providecommand{\figdir}{./Fig}
\providecommand{\fig@file}[1]{\@pathname/\figdir/#1}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Plot commands
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\DeclareRobustCommand{\fullplot}[4][htbp]{%
\begin{figure*}[#1]
\centering
\includegraphics[#3]{\fig@file{#2}.\ext}
\caption[]{#4}
\label{fig:#2}
\end{figure*}}

\DeclareRobustCommand{\sideplot}[4][htbp]{%
\begin{figure}[#1]
\centering
\includegraphics[#3]{\fig@file{#2}.\ext}
\caption[]{#4}
\label{fig:#2}
\end{figure}}

\def\plot{\@ifstar{\fullplot}{\sideplot}}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Cite commands - SEP style
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\let\sep@cite\cite
\def\cite{\sep@cite[]}
\def\longcite{\sep@cite}
\newcommand{\shortcite}[1]{(\citeyear{#1})}

\DeclareRobustCommand{\refname}{REFERENCES}
\renewcommand{\bibsection}{\section{\refname}}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Define SEG-style vectors and tensors
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\newcommand{\bold}[1]{\ensuremath\mathbf{#1}}
\let\sv\bold
\let\realbold\bold
\newcommand{\st}[1]{\uvec{\realbold{#1}}}

