#include <sitedef.h>
#if defined (HAVE_MOTIF) || defined(HAVE_ATHENA)
/*
data object code
data object consists of axes, buffers, and dataset info
four input data formats recognized:
	FORMAT	CONDITIONS		ACTIONS
	vgrid	DataGridHeader()==1	DataLoadFloat() or DataLoadByte()
	segy	else segy=1		DataGetpar(), DataLoadFloat()
	floats	else float=1		DataGetpar(), DataLoadFloat()
	bytes	else			DataGetpar(), DataLoadByte()
*/
/* Modified:
   Hector Urdaneta 7/26/96 
		   Had to bring up in the code the line 
                   data->value_size = DATA_VALUE_SIZE;
                   since data->value_size was being used in the 
                   subroutine GetDatapar without initialized
*/
#include <stdio.h>
#include "rick.h"
Datalist datalist;
void vel_init();
void nmoed_init(Data nmoed);
void vscan_init(Data vscan);

/* initialize dataset from getpar */
void DataInit(){
int i,ierr;
char temp_ch[1024],temp2_ch[1024];

	NEW(Datalist,datalist,1);
	for (i=0; i < NUM_VIEWS; i++) datalist->data[i]=0;

  /*next we  need to see how many datasets we need to load */
	ierr=1;datalist->ndata=1;
	datalist->data[0]=InitData("in",0);
	while(ierr==1 && datalist->ndata < NUM_VIEWS){
		sprintf(temp_ch,"data%d",datalist->ndata);
		ierr=rick_getch(temp_ch,"s",temp2_ch);
		if(ierr==1){
      /*initialize extra data list */
			datalist->data[datalist->ndata]=InitData(temp_ch,datalist->ndata);
			CheckData();
			datalist->ndata++;
		}
	}
	if(1==rick_getch("mode","s",temp_ch)){
		if(0==strcmp("velan",temp_ch)){
			vel_init();
		}
	}
}

void vel_init(){
Data data,vscan,nmoed;
float o,d,km,tempr,ov,dv;
int n,nv,n1,n2,n3,n4,n5;
string label;


data=datalist->data[0];
NEW(Data,datalist->data[datalist->ndata],1);
vscan=datalist->data[datalist->ndata]; datalist->ndata++;
NEW(Data,datalist->data[datalist->ndata],1);
nmoed=datalist->data[datalist->ndata]; datalist->ndata++;

  tempr=AxisValue(data->axis[DATA_AXIS2],0)-AxisValue(data->axis[DATA_AXIS2],AxisSize(data->axis[DATA_AXIS2]));
  if(tempr < -100. || tempr > 100.) km=1.;
  else km=1000.;

  /*grab semblance parameters */
  if(0==rick_getch("v0","f",&ov)) ov=1.5*km;
  if(0==rick_getch("nv","d",&nv)) nv=50;
  if(0==rick_getch("dv","f",&dv)) dv=(3.5*km-ov)/(nv-1);

  n1=AxisSize(data->axis[1]);
  n2=AxisSize(data->axis[2]);
  n3=AxisSize(data->axis[3]);
  n4=AxisSize(data->axis[4]);
  n5=AxisSize(data->axis[5]);


/*initialize semblance view */
  vscan->value_size = DATA_VALUE_SIZE;
  vscan->esize = 4;
  vscan->hbytes = 0; vscan->size=1;
  vscan->axis[DATA_AXIS1]= CopyAxis(DATA_AXIS1,1,data->axis[DATA_AXIS1]);
  vscan->size=AxisSize(vscan->axis[DATA_AXIS1]);

  vscan->axis[DATA_AXIS2]= AxisInit3 (DATA_AXIS2,vscan->size,"my",nv,ov,dv,1.);
  vscan->size=vscan->size*AxisSize(vscan->axis[DATA_AXIS2]);

  vscan->axis[DATA_AXIS3]= CopyAxis(DATA_AXIS3,n1*nv,data->axis[DATA_AXIS3]);
  vscan->axis[DATA_AXIS4]= CopyAxis(DATA_AXIS4,n1*nv*n3,data->axis[DATA_AXIS4]);
  vscan->axis[DATA_AXIS5]= CopyAxis(DATA_AXIS5,n1*nv*n3*n4,data->axis[DATA_AXIS5]);




/*initialize semblance view */
  nmoed->value_size = DATA_VALUE_SIZE;
  nmoed->esize = 4;
  nmoed->hbytes = 0; nmoed->size=1;
  nmoed->axis[DATA_AXIS1]= CopyAxis(DATA_AXIS1,1,data->axis[DATA_AXIS1]);
  nmoed->size=AxisSize(nmoed->axis[DATA_AXIS1]);

  nmoed->axis[DATA_AXIS2]= CopyAxis(DATA_AXIS2,nmoed->size,data->axis[DATA_AXIS2]);
  nmoed->size=nmoed->size*AxisSize(nmoed->axis[DATA_AXIS2]);

  nmoed->axis[DATA_AXIS3]= CopyAxis(DATA_AXIS3,n1*n2,data->axis[DATA_AXIS3]);
  nmoed->axis[DATA_AXIS4]= CopyAxis(DATA_AXIS4,n1*n2*n3,data->axis[DATA_AXIS4]);
  nmoed->axis[DATA_AXIS5]= CopyAxis(DATA_AXIS5,n1*n2*n3*n4,data->axis[DATA_AXIS5]);

  nmoed->axis[DATA_VALUE] = AxisInit2(DATA_VALUE,1,label,nmoed->value_size,
  nmoed->low,(nmoed->high-nmoed->low)/(nmoed->value_size-1),1);
  vscan->axis[DATA_VALUE] = AxisInit2(DATA_VALUE,1,label,vscan->value_size,
  vscan->low,(vscan->high-vscan->low)/(vscan->value_size-1),1);

  vscan->pickable=1;
  nmoed->pickable=0;
  vscan->data_2d=1;
  nmoed->data_2d=1;
	data->pickable=0;
	strcpy(nmoed->tag,"nmoed");
	strcpy(vscan->tag,"vscan");
}


void float_to_byte(float *input, Buffer output, int n1, float *tgain, register float bias, register float scale, int value_size){
int i1 ;
register float  *fp, *gp,*fe;
register byte dp;
register int datum;


  dp=output;


    for (fp=input, fe=fp+n1, gp=tgain; fp<fe; fp++, dp++, gp++) {
      datum = (*fp* *gp + bias) * scale;
      datum = datum > 0 ? datum : 0;
      datum = datum < value_size ? datum : value_size-1;
      datum += DATA_VALUE_BASE;
      *dp = datum;
      }
}

void compute_gain_pars(Data data,  float *buf2, int n, register float *scale, register float *bias ){

  if (data->low == DATA_LOW && data->high == DATA_HIGH) {
    string label;
    data->min = DataCent (buf2,n,DATA_CENT_MIN);
    data->low = DataCent (buf2,n,DATA_CENT_LOW);
    data->high = DataCent (buf2,n,DATA_CENT_HIGH);
    data->max = DataCent (buf2,n,DATA_CENT_MAX);

    strcpy (label,AxisLabel(data->axis[DATA_VALUE]));
    FREE (data->axis[DATA_VALUE]);
    data->axis[DATA_VALUE] = AxisInit2(DATA_VALUE,1,label,data->value_size+1,
    data->low,(data->high-data->low)/data->value_size,1);
    }
  if (data->low == 0.0 && data->high == 0.) {
    err ("first panel appears to be all zeros; set clip");
    }
  if (data->high > data->low) {
    *bias = -data->low;
    *scale = data->value_size / (data->high - data->low);
    }
  else  {
    *bias = 0.;
    *scale = 0.5 * data->value_size / data->high;
    }

}


void CheckData(){
int i,ndata;
ndata=datalist->ndata;
for(i=DATA_AXIS1; i<= DATA_AXIS5; i++){
 if(datalist->data[0]->axis[i]->size != datalist->data[ndata]->axis[i]->size)
		seperr("Size of axis(%d) do not match for input=%d and data%d=%d \n",
       i,datalist->data[0]->axis[i]->size,ndata, 
          datalist->data[ndata]->axis[i]->size);
 if(datalist->data[0]->axis[i]->delta != datalist->data[ndata]->axis[i]->delta)
		fprintf(stderr,"WARNING: d of axis(%d) do not match for input=%f and data%d=%f \n",
       i,datalist->data[0]->axis[i]->delta,ndata, 
          datalist->data[ndata]->axis[i]->delta);
 if(datalist->data[0]->axis[i]->first != datalist->data[ndata]->axis[i]->first)
		fprintf(stderr,"WARNING: o of axis(%d) do not match for input=%f and data%d=%f \n",
       i,datalist->data[0]->axis[i]->delta,ndata, 
          datalist->data[ndata]->axis[i]->delta);
}
   
}

Data InitData (char *tag, int data_num)
	{
	Data data;
	Axis axis;
	int iaxis, i, vfd, ncolor;
	FILE *fd;
	char extra[1024];

  if(data_num>0) sprintf(extra,"%d",data_num);
	else strcpy(extra,"");

	NEW(Data,data,1);
	data->value_size = DATA_VALUE_SIZE;
		DataGetpar (data,tag,extra);
	strcpy (data->title,"stdin");
	if (rick_auxfetch("title","s",data->title,tag,extra) == 0) 
		rick_auxfetch("in","s",data->title,tag,extra);

	data->tpow = 0.;
	rick_auxfetch ("tpow","f",&data->tpow,tag,extra);
	data->gpow = 1.;
	rick_auxfetch ("gpow","f",&data->gpow,tag,extra);
	data->transp = 0;
	rick_auxfetch("transp","d",&data->transp,tag,extra);
	/* perform transpose by swapping axes */
	if (data->transp) {
		NEW (Axis,axis,1);
		axis[0] = data->axis[DATA_AXIS1][0];
		data->axis[DATA_AXIS1][0] = data->axis[DATA_AXIS2][0];
		data->axis[DATA_AXIS2][0] = axis[0];
		FREE (axis);
		}
	data->value_base = DATA_VALUE_BASE;
	strcpy(data->tag,tag);
	if(data->esize==4) 
     data->value_size = DrawNcolor() - data->value_base;

	strcpy (data->veldata,"");
	rick_auxfetch ("veldata","s",data->veldata,tag,extra);
	NEW(Buffer,data->buffer,data->size);
	data->grab_buffer=return_buffer;
	data->pickable=1;
	return (data);
	}



/* fetch grid parameters from getpar */
DataGetpar (Data data,char *tag, char *extra)
	{
	int iaxis;
	string label;

	data->segy = DATA_SEGY;
	rick_auxfetch ("segy","d",&data->segy,tag,extra);
	data->esize = 0;  rick_auxfetch ("esize","d",&data->esize,tag,extra);
	if (data->segy) data->esize = 4;
	else if (data->esize == 0) data->esize = 1;
	data->size = 1;
	data->hbytes = 0;

	for (iaxis=DATA_AXIS1; iaxis<=DATA_AXIS3; iaxis++) {
		/* accumulating size are axis strides */
		data->axis[iaxis] = AxisInit (tag,iaxis,data->size);
		data->size *= AxisSize (data->axis[iaxis]);
		}

	if(AxisSize(data->axis[DATA_AXIS3])==1) data->data_2d=1;
	else data->data_2d=0;

	data->axis[DATA_AXIS4] = AxisInit (tag,4,data->size);
	data->axis[DATA_AXIS5] = AxisInit (tag,5,data->size);
	data->size *= AxisSize (data->axis[DATA_AXIS4]);
	data->size *= AxisSize (data->axis[DATA_AXIS5]);
	data->max = DATA_HIGH;
	rick_auxfetch ("max","f",&data->max,tag,extra);
	data->high = data->max;
	rick_auxfetch ("high","f",&data->high,tag,extra);
	rick_auxfetch ("pclip","f",&data->high,tag,extra);
	rick_auxfetch ("clip","f",&data->high,tag,extra);
	if (data->high != DATA_HIGH) {
		data->min = -data->high;
		data->low = data->min;
		}
	else	{
		data->min = DATA_LOW;
		data->low = data->min;
		}
	rick_auxfetch ("low","f",&data->low,tag,extra);
	rick_auxfetch ("nclip","f",&data->low,tag,extra);
	rick_auxfetch ("min","f",&data->min,tag,extra);
	data->cent = DATA_CENT;
	rick_auxfetch ("perc","f",&data->cent,tag,extra);
	rick_auxfetch ("cent","f",&data->cent,tag,extra);
	strcpy (label,"samples"); rick_auxfetch ("value","s",label,tag,extra);
	data->axis[DATA_VALUE] = AxisInit2(DATA_VALUE,1,label,data->value_size,
	data->low,(data->high-data->low)/(data->value_size-1),1);
	return(0);
}

/* decide which data reading routine to use */
void DataLoad ()
	{
	int i;
	char tag[1024],add_to[1024];

	for(i=0; i < datalist->ndata; i++){
		if(0==strcmp(datalist->data[i]->tag,"vscan")){
        vscan_init(datalist->data[i]);}
		else if(0==strcmp(datalist->data[i]->tag,"nmoed")){
        nmoed_init(datalist->data[i]);
    }
		else{
			switch (datalist->data[i]->esize) {
				case 4: DataLoadFloat (datalist->data[i],datalist->data[i]->tag); break;
				case 1: DataLoadByte (datalist->data[i],datalist->data[i]->tag); break;
			}
			if(i>0) sprintf(add_to,"%d",i);
			else strcpy(add_to,"");
	    if(1==rick_auxgetch("pick","s",datalist->data[i]->pik->file,add_to))
				PikRead(datalist->data[i]);
		}
	}
	UIMessage ("data loaded");
}

void nmoed_init(Data nmoed){
Data data;
float o,d,km,tempr,ov,dv;
int n,nv;
string label;


data=datalist->data[0];

  /*initialize cmp view*/

  nmoed->max = DATA_HIGH;
  nmoed->high = nmoed->max;
  nmoed->min = DATA_LOW;
  nmoed->low = nmoed->min;
  nmoed->cent = data->cent;
  strcpy (label,"samples"); 
  strcpy (nmoed->title,"Nmo Result");

  nmoed->tpow = data->tpow;
  nmoed->gpow = data->gpow;
  nmoed->transp = data->transp;
  nmoed->value_base = data->value_base;

  NEW(Buffer,nmoed->buffer,nmoed->size);
  nmoed->grab_buffer=return_buffer;
  SetAxisStride(nmoed->axis[3],0);
  SetAxisStride(nmoed->axis[4],0);
  SetAxisStride(nmoed->axis[5],0);
  semb_init();
}

Data DataFromTag(char *tag){
int i;

i=DataNumFromTag(tag);
if(i==-1) seperr("Unable to find dataset %s \n",tag);
	return(datalist->data[i]);
}

int DataNumFromTag(char *tag){
int i;

for(i=0; i < ReturnNdata(); i++){
	if(strcmp(datalist->data[i]->tag,tag)==0) return(i);
}

return(-1);


}

void vscan_init(Data vscan){
Data data;
float o,d,km,tempr,ov,dv;
int n,nv;
string label;


data=datalist->data[0];

  vscan->max = DATA_HIGH;
  vscan->high = vscan->max;
  vscan->min = DATA_LOW;
  vscan->low = vscan->min;
  vscan->cent = DATA_CENT;
  strcpy (label,"samples"); 
  strcpy (vscan->title,"Semblance");
  vscan->tpow = 0.;
  vscan->gpow = 1.;
  vscan->transp = 0;
  vscan->value_base = DATA_VALUE_BASE;

NEW(Buffer,vscan->buffer,vscan->size);
  vscan->grab_buffer=return_buffer;


  SetAxisStride(vscan->axis[3],0);
  SetAxisStride(vscan->axis[4],0);
  SetAxisStride(vscan->axis[5],0);
}





/* load byte data; compress to 7 bits- 0-128 */
void DataLoadByte (Data data,char *tag)
	{
	byte table;
	int i, value, ncolor,n12;
	register byte tp;
	register Buffer bp, be;

	/* read data */
	UIMessage ("loading byte data ...");
	if(1==data_is_2d(data)){
		n12=data->size/2;
		sreed(tag,data->buffer,n12);	
		memcpy((void *)(data->buffer + n12),(const void*) data->buffer,n12);
	}
	else sreed (tag,data->buffer,data->size);

	/* compute color compression table table */
	NEW (byte,table,256);
	/* bounds */
	table[0] = 0;
	/* compute lookup table */
	ncolor = ColorSize();
	ncolor = DataValueSize(data);
	for (i=0; i<256; i++) {
		table[i] = (i * ncolor) / 255;
		}
	for (bp=data->buffer, be=bp+data->size, tp=table; bp<be; bp++) {
		*bp = tp[*bp];
		}
	DataComputeHistogram (data);
	FREE (table);
	}


/* load and convert float or segy data */
void DataLoadFloat (Data data,char *tag)
	{
	int i1, i2, i3, n1, n2, n3, base, size, head;
	float *buf1, *buf2,  *trace, *tgain;
	register float scale, bias, *bp1, *bp2, *fp, *fe, *gp;
	register int datum;
	register byte dp;
	double pow(), log();

	if (data->segy) UIMessage ("loading segy data ...");
	else UIMessage ("loading float data ...");
	/* calculate gain parameters: clip, gpow */
	n1 = AxisSize (data->axis[DATA_AXIS1]);
	n2 = AxisSize (data->axis[DATA_AXIS2]);
	n3 = AxisSize (data->axis[DATA_AXIS3])
	   * AxisSize (data->axis[DATA_AXIS4])
	   * AxisSize (data->axis[DATA_AXIS5]);
	if (data->segy) head = 60;
	else head = 0;
	NEW (float*,buf1,n1*n2);
	NEW (float*,buf2,n1*n2);
	NEW (float*,tgain,n1);
	size = (n1+head)>DATA_HEAD1?(n1+head):DATA_HEAD1;
	size = size > data->hbytes ? size : data->hbytes;
	NEW (float*,trace,size);
	fe = trace + n1 + head;
	bp1 = buf1;
	bp2 = buf2;
	/* time gain vector */
	for (i1=0; i1<n1-1; i1++) {
		tgain[i1] = AxisValue(data->axis[DATA_AXIS1],i1+1);
		tgain[i1] = tgain[i1] != 0.0 ? pow(tgain[i1],data->tpow) : 1.0;
		}
	tgain[n1-1] = tgain[n1-2];
	/* remember first panel in buf1; copy for percentile */
	for (i2=0; i2<n2; i2++) {
		sreed (tag,trace,(n1+head)*sizeof(trace[0]));
		for (fp=trace+head, fe=fp+n1, gp=tgain; fp<fe; fp++, bp1++, bp2++, gp++) {
			*bp1 = *fp * *gp;
			*bp2 = *bp1;
			}
		}
	if (data->low == DATA_LOW && data->high == DATA_HIGH) {
		string label;
		data->min = DataCent (buf2,n1*n2,DATA_CENT_MIN);
		data->low = DataCent (buf2,n1*n2,DATA_CENT_LOW);
		data->high = DataCent (buf2,n1*n2,DATA_CENT_HIGH);
		data->max = DataCent (buf2,n1*n2,DATA_CENT_MAX);
		strcpy (label,AxisLabel(data->axis[DATA_VALUE]));
		FREE (data->axis[DATA_VALUE]);
		data->axis[DATA_VALUE] = AxisInit2(DATA_VALUE,1,label,data->value_size+1,
		data->low,(data->high-data->low)/data->value_size,1);
		}
	FREE (buf2);
	if (data->low == 0.0 && data->high == 0.) {
		err ("first panel appears to be all zeros; set clip");
		}
	if (data->high > data->low) {
		bias = -data->low;
		scale = data->value_size / (data->high - data->low);
		}
	else	{
		bias = 0.;
		scale = 0.5 * data->value_size / data->high;
		}
	dp = data->buffer;
	for (fp=buf1, fe=buf1+n1*n2; fp<fe; fp++, dp++) {
		datum = (*fp + bias) * scale;
		datum = datum > 0 ? datum : 0;
		datum = datum < data->value_size ? datum : data->value_size-1;
		datum += DATA_VALUE_BASE;
		*dp = datum;
		}
	FREE (buf1);
	/* convert rest of traces */
	for (i3=1; i3<n3; i3++)
	for (i2=0; i2<n2; i2++) {
		if (sreed (tag,trace,(n1+head)*sizeof(trace[0]))!=(n1+head)*sizeof(trace[0])) break;
		for (fp=trace+head, fe=fp+n1, gp=tgain; fp<fe; fp++, dp++, gp++) {
			datum = (*fp* *gp + bias) * scale;
			datum = datum > 0 ? datum : 0;
			datum = datum < data->value_size ? datum : data->value_size-1;
			datum += DATA_VALUE_BASE;
			*dp = datum;
			}
		}
	DataComputeHistogram (data);
	FREE (trace);
	FREE (tgain);
	}

/*      percentile subroutine
 *      based on Canales, SEP-10
 *      p - percentile <0.,99.999999999999>
 *      x - data
 *      n - vector length
 *	this routine changes data order, so sort a copy
 */
float DataCent (float *x,int n,float p)
        {
        int q;
        register float *i, *j, ak;
        float *low, *hi, buf, *k;
	p = p < 99.999 ? p : 99.999;
	p = p > 0.0 ? p : 0.0;
	q = (p * n) / 100.;
	if (q == n) q--;
        for (low=x, hi=x+n-1, k=x+q; low<hi;)
                {
                ak = *k;
                i = low; j = hi;
                do
                        {
                        while (*i < ak) i++;     
                        while (*j > ak) j--;     
                        if (i<=j)     
                                {
                                buf = *i;
                                *i++ = *j;
                                *j-- = buf;
                                }
                        } while (i<=j);
                if (j<k) low = i; if (k<i) hi = j;
                }
        return (*k);
        }

/* return long data name */
char* DataLabel (Data data)
	{
	static Message message;

	if (!data) return (0);
	sprintf (message,"%s: %dx%dx%dx%dx%d=%d %s samples",data->title,
		AxisSize(data->axis[DATA_AXIS1]),
		AxisSize(data->axis[DATA_AXIS2]),
		AxisSize(data->axis[DATA_AXIS3]),
		AxisSize(data->axis[DATA_AXIS4]),
		AxisSize(data->axis[DATA_AXIS5]),
		data->size,
		AxisLabel(data->axis[DATA_VALUE]));
	return (message);
	}

/* return short data name */
char* DataTitle (Data data)
	{
	if (!data) return (0);
	return (data->title);
	}

/* return data file */
char* DataFile (Data data)
	{
	if (!data) return (0);
	return (data->file);
	}

/* return short name of dataset from filename */
char* DataShortName (Data data)
	{
	static string name;
	char *c;

	/* truncate leading pathname */
	if ((c = strrchr (data->file,'/')) != NULL) strcpy (name,c+1);
	else strcpy (name,data->file);
	/* truncate trailing tail */
	if ((c = strchr (name,'.')) != NULL) *c = '\0';
	return (name);
	}
/* return data buffer */
Buffer DataBuffer (Data data)
	{
	if (!data) return (0);
	return (data->grab_buffer(data));
	}

/* return axis reference */
Axis DataAxis (Data data,int iaxis)
	{
	if (!data || iaxis>DATA_NAXIS) return (0);
	return (data->axis[iaxis]);
	}

/* return data size */
int DataSize (Data data)
	{
	return (data->size);
	}

/* return longest dimension */
DataMaxDim (Data data)
	{
	int n[DATA_NAXIS], max, iaxis;

	if (!data) return (0);
	for (iaxis=0, max=1; iaxis<DATA_NAXIS-1; iaxis++) {
		max = AxisSize(data->axis[iaxis+1]) > max ? AxisSize(data->axis[iaxis+1]) : max;
		}
	return (max);
	}

/* return data amplitude */
float DataValue (Data data,int value)
	{
	return (data->low+((data->high-data->low)*((value&0x7F)-data->value_base))/data->value_size);
	}

/* return data value size */
int DataValueSize (Data data)
	{
	if (!data) return (0);
	else return (data->value_size);
	}

/* return base data value */
int DataValueBase (Data data)
	{
	if (!data) return (0);
	else return (data->value_base);
	}


/* return data high clip */
float DataHigh ()
	{
	View myv;

	myv=current_view();
	if (!myv->data) return (0.0);
	return (myv->data->high);
	}

/* return data low clip */
float DataLow ()
	{
	View myv;

	myv=current_view();

	if (!myv->data) return (0.0);
	return (myv->data->low);
	}

int ReturnNdata(){
return(datalist->ndata);
}

/* return integer percentage of histogram */
int DataHistogram (int i)
	{
	View myv;

	myv=current_view();

	if (!myv->data || i<0 || i>255) return (0);
	return ((int)(100*myv->data->histogram[i]));
	}

void DataComputeHistogram (Data data)
	{
	register Buffer bp, be;
	register float *hp;
	int i;
	float max;

	for (i=0; i<256; i++) {
		data->histogram[i] = 0;
		}
	for (bp=data->buffer,be=bp+data->size,hp=data->histogram; bp<be; bp++) {
		hp[*bp] += 1.;
		}
	for (i=0, max=0, data->hist_low=255, data->hist_high=0; i<256; i++) {
		data->histogram[i] = data->histogram[i] / data->size;
		max = data->histogram[i] > max ? data->histogram[i] : max;
		if (data->hist_low == 255 && data->histogram[i] != 0) data->hist_low = i;
		if (data->histogram[i] != 0) data->hist_high = i;
		}
	max = max > 0. ? 1. / max : 0.;
	for (i=0; i<256; i++) {
		data->histogram[i] = data->histogram[i] * max;
		}
	}

/* print data information */
int DataInfo (Data data)
	{
	Message message;

	sprintf (message,"Data: %s: in=%s %dx%dx%dx%dx%d=%d esize=%d segy=%d hbytes=%d veldata=%s script=%s",
		data->title,data->file,
		AxisSize(data->axis[DATA_AXIS1]),
		AxisSize(data->axis[DATA_AXIS2]),
		AxisSize(data->axis[DATA_AXIS3]),
		AxisSize(data->axis[DATA_AXIS4]),
		AxisSize(data->axis[DATA_AXIS5]),
		data->size,
		data->esize,
		data->segy,
		data->hbytes,
		data->veldata,
		data->script);
	UIMessage (message);
  return(0);
	}

/* data value parameters */
int DataValueInfo (Data data)
	{
	Message message;

	sprintf (message, "Value: min=%g low=%g high=%g max=%g tpow=%g gpow=%g base=%d size=%d",
		data->min,
		data->low,
		data->high,
		data->max,
		data->tpow,
		data->gpow,
		data->value_base,
		data->value_size);
	UIMessage (message);
  return(0);
	}

/* save data parameters */
int DataSavePar (Data data)
	{
	int iaxis;
	Message message;

	if (!data) return(-1);
	sprintf (message,"Data: %s: in=%s %dx%dx%dx%dx%d=%d esize=%d segy=%d hbytes=%d script=%s",
		data->title,
		data->file,
		AxisSize (data->axis[DATA_AXIS1]),
		AxisSize (data->axis[DATA_AXIS2]),
		AxisSize (data->axis[DATA_AXIS3]),
		AxisSize (data->axis[DATA_AXIS4]),
		AxisSize (data->axis[DATA_AXIS5]),
		data->size,
		data->esize,
		data->segy,
		data->hbytes,
		data->script);
	UISaveMessage (message);
	sprintf (message, "Value: min=%g low=%g high=%g max=%g tpow=%f gpow=%g base=%d size=%d",
		data->min,
		data->low,
		data->high,
		data->max,
		data->tpow,
		data->gpow,
		data->value_base,
		data->value_size);
	UISaveMessage (message);
	for (iaxis=DATA_AXIS1; iaxis<=DATA_AXIS5; iaxis++) {
		AxisSavePar (data->axis[iaxis]);
		}
  return(0);
	}

/* dump data bytes */
int DataDumpBytes (Data data,char *file,int fd)
	{
	string parfile, datafile;
	register byte bp, be;
	Message message;

	DrawWatch(1);
	for (bp=data->buffer, be=bp+data->size; bp<be;) *bp++ <<= 1;
	write (fd,data->buffer,data->size);
	for (bp=data->buffer, be=bp+data->size; bp<be;) *bp++ >>= 1;
	DataDumpHeader (data,file,fd,1);
	DrawWatch(0);
  return(0);
	}

/* dump data vgrid header */
int DataDumpHeader (Data data,string file,int datafd,int esize)
	{
	Message message;
	char parfile[80];
	extern FILE *savefd;

	if (data->gh.dtype == 0.) { /* build a header */
		data->gh.scale = 1.;
		data->gh.n1 = AxisSize (data->axis[DATA_AXIS1]) * data->gh.scale;
		data->gh.n2 = AxisSize (data->axis[DATA_AXIS2]) * data->gh.scale;
		data->gh.n3 = AxisSize (data->axis[DATA_AXIS3]) * data->gh.scale;
		data->gh.n4 = AxisSize (data->axis[DATA_AXIS4]) * data->gh.scale;
		data->gh.n5 = AxisSize (data->axis[DATA_AXIS5]) * data->gh.scale;
		data->gh.d1 = AxisDelta (data->axis[DATA_AXIS1]) * data->gh.scale;
		data->gh.d2 = AxisDelta (data->axis[DATA_AXIS2]) * data->gh.scale;
		data->gh.d3 = AxisDelta (data->axis[DATA_AXIS3]) * data->gh.scale;
		data->gh.d4 = AxisDelta (data->axis[DATA_AXIS4]) * data->gh.scale;
		data->gh.d5 = AxisDelta (data->axis[DATA_AXIS5]) * data->gh.scale;
		data->gh.o1 = AxisFirst (data->axis[DATA_AXIS1]) * data->gh.scale;
		data->gh.o2 = AxisFirst (data->axis[DATA_AXIS2]) * data->gh.scale;
		data->gh.o3 = AxisFirst (data->axis[DATA_AXIS3]) * data->gh.scale;
		data->gh.o4 = AxisFirst (data->axis[DATA_AXIS4]) * data->gh.scale;
		data->gh.o5 = AxisFirst (data->axis[DATA_AXIS5]) * data->gh.scale;
		data->gh.dcdp2 = 1.;
		data->gh.dline3 = 1.;
		data->gh.ocdp2 = 1.;
		data->gh.oline3 = 1.;
		data->gh.gmin = data->low * data->gh.scale;
		data->gh.gmax = data->high * data->gh.scale;
		}
	data->gh.dtype = esize * data->gh.scale;
	write (datafd,&data->gh,sizeof(data->gh));
	close (datafd);
	sprintf (parfile,"%s.H",file);
	if ((savefd = fopen (parfile,"w")) == NULL); UIMessage ("cant create vgrid parfile");
	DataSavePar (data);
	sprintf (message, "Byte data dump into %s",parfile);
	UIMessage (message);
	fclose (savefd);
	return(0);
	}

/* dump data floats */
int DataDumpFloats (Data data,char *file,int fd)
	{
	string parfile, datafile;
	int nn, n3, i;
	Buffer dbuf;
	float *fbuf, min, max;
	Message message;

	DrawWatch (1);
	nn = AxisSize (data->axis[DATA_AXIS1]) * AxisSize (data->axis[DATA_AXIS2]);
	n3 = data->size / nn;
	NEW (float*,fbuf,nn);
	/* convert and dump a plane at a time */
	min = data->high;
	max = data->low;
	for (i=0, dbuf=data->buffer; i<n3; i++, dbuf+=nn) {
		Data2Float (dbuf,fbuf,nn,&min,&max);
		write (fd,fbuf,nn*sizeof(fbuf[0]));
		}
	data->gh.gmin = min * data->gh.scale;
	data->gh.gmax = max * data->gh.scale;
	DataDumpHeader (data,file,fd,4);
	FREE (fbuf);
	DrawWatch (0);
  return(0);
	}

void Data2Float (Buffer dbuf,float *fbuf,int n,float *min,float *max)
	{
	register Buffer bp, be;
	register float *fp, *mp, *fe, Min, Max;
	View myv;
	myv=current_view();
	Min = *min;
	Max = *max;
	/* convert */
	for (mp=AxisValues(myv->data->axis[DATA_VALUE])-DATA_VALUE_BASE, bp=dbuf, fp=fbuf, be=dbuf+n; bp<be; bp++, fp++) {
		*fp = mp[*bp];
		Max = Max > *fp ? Max : *fp;
		Min = Min < *fp ? Min : *fp;
		}
	*min = Min;
	*max = Max;
	}

int data_is_2d(Data data){
return(data->data_2d);
}

void set_data_lists(){
extern Pikers pikers;
extern Anoviews anoviews;
int i;
for(i=0; i < datalist->ndata;i++){
	datalist->data[i]->pik=pikers->pik[i];
	datalist->data[i]->ano=anoviews->ano[i];
}
}
Buffer return_buffer(Data data){

return(data->buffer);

}

int rint2 (float x) 
	{
	int i;

	i = x;
	return ((x-i) < (1-(x-i)) ? i : i+1);
	}
#endif
