#define SET_SDOC 1
/*
-------------------------------------------------

Author: Robert Clapp, ESMB 463, 7230253

Date Created:Thu Jan  8 08:20:53 PST 2004

Purpose: 

*/	 

#include "superset_internal.h"
#include <superset.h> 


/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
/*
<
sep3d_set_coord_vals

USAGE
ierr= sep3d_set_coord_vals(char *sep3dname,  int *coords)

INPUT PARAMETERS
sep3dname -  char*   tag associated with sep3d structure

OUTPUT PARAMETERS
coords       -  int*    coordinate value (ndim-1 * ncoord)

RETURN VALUES
0   =  if it works correctly
(check superset.h for other return values and there meaning)

DESCRIPTION
Set the coordinate values for the current in block
>
*/


_XFUNCPROTOBEGIN
int sep3d_grab_ncoord(char *sep3dname,int *ncoord)
_XFUNCPROTOEND
{
sep_3d *info;

info = tag_info_sep3d(sep3dname, INQUIRE);  /* get info on this tag */
if(info == SEPNULL)
  return (sepwarn(INVALID_STRUC,"tag:%s  invalid struc\n",sep3dname));

*ncoord=info->ncoord;

return(SUCCESS);
}
_XFUNCPROTOBEGIN
int sep3d_set_coord_vals(char *sep3dname,int *coords)
_XFUNCPROTOEND
{
int i1,ierr,idim,i;
sep_3d *info;
double *block;


info = tag_info_sep3d(sep3dname, INQUIRE);  /* get info on this tag */
if(info == SEPNULL)
  return (sepwarn(INVALID_STRUC,"tag:%s  invalid struc\n",sep3dname));

if(info->coord==SEPNULL) 
 return(sepwarn(NOT_MET, "coords have not been allocated (%s)\n", sep3dname));

block=(double*) malloc(sizeof(double)*(info->ndims-1));
for(idim=1,block[0]=1; idim < info->ndims-1; idim++) 
  block[idim]=block[idim-1]*info->n[idim];

for(i1=0,i=0; i1 < info->ncoord; i1++){
  info->coord[i1]=0;
  for(idim=0; i1 < info->ndims-1; idim++,i++)
    info->coord[i1]+=(double)(coords[i]-1)*block[idim];
}

free(block);
return(SUCCESS);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
/*
<
sep3d_grab_coord_vals

USAGE
ierr= sep3d_grab_coord_vals(char *sep3dname,  int *coords)

INPUT PARAMETERS
sep3dname -  char*   tag associated with sep3d structure

OUTPUT PARAMETERS
coords       -  int*    coordinate (ndim-1*ncoord)

RETURN VALUES
0   =  if it works correctly
(check superset.h for other return values and there meaning)

DESCRIPTION
Find the data record number of the dataset
>
*/


_XFUNCPROTOBEGIN
int sep3d_grab_coord_vals(char *sep3dname,int *coords)
_XFUNCPROTOEND
{
int i1,ierr,i,idim,j;
sep_3d *info;
double *block,*n;


info = tag_info_sep3d(sep3dname, INQUIRE);  /* get info on this tag */
if(info == SEPNULL)
  return (sepwarn(INVALID_STRUC,"tag:%s  invalid struc\n",sep3dname));




if(info->coord==SEPNULL)
	return(sepwarn(NOT_MET,"coord has not been set in %s\n",sep3dname));

block=(double*) malloc(sizeof(double)*(info->ndims-1));
n=(double*) malloc(sizeof(double)*(info->ndims-1));
for(idim=1,block[0]=1; idim < info->ndims-1; idim++) {
  block[idim]=block[idim-1]*info->n[idim];
  n[idim]=info->n[idim];
}

for(i1=0,i=0; i1 < info->ncoord; i1++){
  for(idim=0; idim < info->ndims-1; idim++,i++){
/*    j=(int)((info->coord[i1]/block[idim])/(double)n[idim]);*/
/*    coords[i]=*/
/*     (int)((double)(info->coord[i1]/block[idim])-(double)j*(double)n[idim]);*/
      coords[i]=((int)(info->coord[i1]/block[idim]))%info->n[idim+1]+1;
  }

}
free(block); free(n);
return(SUCCESS);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
/*
<
sep3d_set_coordh

USAGE
ierr= sep3d_set_coordh(char *sep3dname,  int *coords, int *coordb, int block)

INPUT PARAMETERS
sep3dname -  char*   tag associated with sep3d structure

OUTPUT PARAMETERS
coords       -  int*    coordinate value part1

coordb       -  int*    coordinate value part2

block        -  int     block (coord=coordb*block+coords)

RETURN VALUES
0   =  if it works correctly
(check superset.h for other return values and there meaning)

DESCRIPTION
Set the coordinate values for the current in block
>
*/


_XFUNCPROTOBEGIN
int sep3d_set_coordh(char *sep3dname,int *coords, int *coordb, int block)
_XFUNCPROTOEND
{
int i1,ierr;
sep_3d *info;


info = tag_info_sep3d(sep3dname, INQUIRE);  /* get info on this tag */
if(info == SEPNULL)
  return (sepwarn(INVALID_STRUC,"tag:%s  invalid struc\n",sep3dname));

if(info->coord==SEPNULL) 
 return(sepwarn(NOT_MET, "coords have not been allocated (%s)\n", sep3dname));

for(i1=0; i1 < info->ncoord; i1++)info->coord[i1]=coords[i1]+coordb[i1]+block;
return(SUCCESS);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
/*
<
sep3d_grab_coord

USAGE
ierr= sep3d_grab_coordh(char *sep3dname,  int *coords, int *coordb, int *block)

INPUT PARAMETERS
sep3dname -  char*   tag associated with sep3d structure

OUTPUT PARAMETERS
coords       -  int*    coord part 1 of dataset

coordb       -  int*    coord part 2 of dataset

block        -  int*    block (coord=coords+block*coordb)

RETURN VALUES
0   =  if it works correctly
(check superset.h for other return values and there meaning)

DESCRIPTION
Find the data record number of the dataset
>
*/


_XFUNCPROTOBEGIN
int sep3d_grab_coordh(char *sep3dname,int *coords, int *coordb, int *block)
_XFUNCPROTOEND
{
int i1,ierr;
sep_3d *info;


info = tag_info_sep3d(sep3dname, INQUIRE);  /* get info on this tag */
if(info == SEPNULL)
  return (sepwarn(INVALID_STRUC,"tag:%s  invalid struc\n",sep3dname));


if(info->coord==SEPNULL)
	return(sepwarn(NOT_MET,"coord has not been set in %s\n",sep3dname));

*block=2000*1000*1000;
 
for(i1=0; i1 < info->ncoord; i1++){
  coordb[i1]=info->coord[i1]/(*block);
  coords[i1]=info->coord[i1]-coordb[i1]*(*block);
}
return(SUCCESS);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
/*
<
SEP3D_delete_coord

USAGE
ierr= SEP3D_delete_coord(sep_3d *info)

INPUT PARAMETERS
info -  sep_3d*   delete coord


RETURN VALUES
0   =  if deletion successful

-1  =  if deletion not successful


DESCRIPTION
Delete coordinate array of the dataset
>
*/

_XFUNCPROTOBEGIN
int SEP3D_delete_coord(sep_3d *info)
_XFUNCPROTOEND
{
int i1,ierr;

if(info->coord!=SEPNULL){
 free(info->coord); info->coord=SEPNULL;
}
info->ncoord=0;
return(SUCCESS);
}


/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
/*
<
SEP3D_alloc_coord

USAGE
ierr= SEP3D_alloc_coord(sep_3d *info, int ncoord)

INPUT PARAMETERS
info -  sep_3d*   structure associated with sep3d structure

ncoord    -  int     size to allocate array


RETURN VALUES
0   =  if allocation successful

-1  =  if allocation not successful


DESCRIPTION
Allocate coordinate array of the dataset
>
*/

_XFUNCPROTOBEGIN
int SEP3D_alloc_coord(sep_3d *info, int ncoord)
_XFUNCPROTOEND
{
int i1,ierr;
SEP3D_delete_coord(info);

info->ncoord=ncoord;
info->coord=(double*)malloc(sizeof(double)*info->ncoord);

return(SUCCESS);
}

int SEP3D_coord_copy(sep_3d *input, sep_3d *output){
int i;

if(input->ncoord>0){
  if(0!=SEP3D_alloc_coord(output,input->ncoord))
    return(sepwarn(NOT_MET,"trouble allocating coord for tag %s \n",output->name));
  memcpy((void*)output->coord,(const void*)input->coord,sizeof(double)*input->ncoord);


return(SUCCESS);

}
}
_XFUNCPROTOBEGIN
int sep3d_copy_coords(char *sep3din,char *sep3dout)
_XFUNCPROTOEND
{
int i1,ierr,idim,i;
sep_3d *in,*out;


in = tag_info_sep3d(sep3din, INQUIRE);  /* get info on this tag */
if(in == SEPNULL)
  return (sepwarn(INVALID_STRUC,"tag:%s  invalid struc\n",sep3din));

out = tag_info_sep3d(sep3dout, INQUIRE);  /* get info on this tag */
if(out == SEPNULL)
  return (sepwarn(INVALID_STRUC,"tag:%s  invalid struc\n",sep3dout));

return(SEP3D_coord_copy(in,out));

}


_XFUNCPROTOBEGIN
int SEP3D_wind_coords(sep_3d *info,int *nsz)
_XFUNCPROTOEND
{
double *blocks,*blockb,*tempd;
int idim,i,n123,ig,ic,nc,iax;
int *have,*mine;

for(i=1,n123=1; i < info->ndims; i++) n123=info->nwind[i]*n123;

if(n123<1){
  *nsz=0;
   return(0);
}
*nsz=n123;


blocks=(double*)malloc(sizeof(double)*(info->ndims-1));
blockb=(double*)malloc(sizeof(double)*(info->ndims-1));
blockb[0]=1;
for(idim=1; idim<info->ndims-1;idim++)
  blockb[idim]=blockb[idim-1]*info->n[idim];
blocks[0]=1;
                                                                                

for(idim=1; idim<info->ndims-1;idim++){
  blocks[idim]=blocks[idim-1]*info->nwind[idim];
}

if(info->dff==SEPNULL){
 if(0!=SEP3D_alloc_coord(info,n123))
    return(sepwarn(NOT_MET,"trouble allocating coord for tag %s \n",
     info->name));
                                                                                
for(i=0; i <info->ncoord;i++) {
    info->coord[i]=0;
    for(idim=1,ig=0; idim< info->ndims; idim++){
      ic=SEPH2C(i,blocks[idim-1],info->nwind[idim]);

      info->coord[i]+=SEPC2H( (ic*info->jwind[idim]+info->fwind[idim]),blockb[idim-1]);
    }
}
}
else{
/*  if(info->dff->sect_axis<=0 || info->dff->nsect ==0 || info->dff->nsect_ithread==0)*/
  if(info->dff->sect_axis<=0 || info->dff->nsect ==0 ) {
      free(blocks);free(blockb);
      return(sepwarn(NOT_MET,"distributed dataset without dff setup completely ERROR tag=%s \n",info->name));
   }
  have=(int*) malloc(sizeof(int)*info->dff->nsect);
  mine=(int*) malloc(sizeof(int)*n123);
  for(i=0; i <info->dff->nsect;i++) have[i]=0;
  for(i=0; i <n123; i++) mine[i]=0;
  for(i=0; i <info->dff->nsect_ithread;i++) {
    have[info->dff->sect_ithread[i]]=1;
 }

  iax=info->dff->sect_axis-1;
  for(i=0,nc=0; i < n123; i++){
    ic=SEPH2C(i,blocks[iax-1],info->nwind[iax])*info->jwind[iax]+info->fwind[iax];
    if(have[info->dff->axis_convert[ic]]==1){ 
        mine[i]=1;nc++;
    }
  }
 free(have);
 if(0!=SEP3D_alloc_coord(info,nc)){
    free(blocks); free(blockb); free(mine); 
    return(sepwarn(NOT_MET,"trouble allocating coord for tag %s \n",
     info->name));
 } 

  for(i=0,nc=0; i < n123; i++){
    if(mine[i]==1){
      info->coord[nc]=0;
      for(idim=1,ig=0; idim< info->ndims; idim++){
        ic=SEPH2C(i,blocks[idim-1],info->nwind[idim]);
        info->coord[nc]+=SEPC2H( (ic*info->jwind[idim]+info->fwind[idim]),blockb[idim-1]);
      }
      nc++;
    }
  }
  free(mine);
}

free(blocks);free(blockb);

return(SUCCESS);
}
