#define SET_SDOC 1
#include<sitedef.h>
#include<string.h>
#include<math.h>
#include<superset.h>
#include<superset_internal.h>
int sep3d_sect_io_verb;
int calc_section_list(char *tag,sep_3d *info,int nsect,char **machs_old,char **tags_old,char **machs_new,char **dff_new,int *nmove,int *move_list,int *move_to);
int sep3d_move_part(char *tag,sep_3d *info, int in_all, int out_all, int nmove, int *move_list,char **tags_in,int *thread_from);


/*
<
USAGE
ierr=sep3d_init_section_par(char *sep3dname, char *dff_base, int axis, char*pattern, int nsect,int master_data)
                                                                                                            
                                                                                                            
INPUT PARAMETERS
sep3dname    -   char*    pointer to the sep_3d structured
dff_base     -   char*    name for the dff tag (most often will be grabbed/created from a tag)
axis         -   int*     Axis to spread over
pattern      -   char*    Distribution pattern (BLOCK or SPREAD)
nsect        -   int*     Number of sections to split the tag into
master_data  -   int      Whether (1) or not (0) the master thread holds any data
                                                                                                            
RETURN VALUES
0    =  if it works correctly
(check superset.h for other return values and there meaning)
                                                                                                            
DESCRIPTION
Creates a dff tag for a dataset 
                                                                                                            
>
*/


_XFUNCPROTOBEGIN
int sep3d_init_section_par(char *sep3dname, char *dff_base, int axis, char *pattern,int nsect, int master_data)
_XFUNCPROTOEND
{
  
  sep_3d *info;
  sep_dff *info_dff;
  int n,i;
  float o,d;
  char label[256],unit[256],temp_ch[256],temp2_ch[25],temp_name[2049];
  char **machnames,**dff_path;
  int *ns, *sect_thread, *axis_convert;
  int last, nremain,isect,impi,istart;



  /*check to make sure this makes sense*/
  info = tag_info_sep3d(sep3dname, INQUIRE);  /* get info on this tag */
  if(info == SEPNULL)
    return (sepwarn(INVALID_STRUC,"tag:%s  invalid struc\n",sep3dname));

  if(info->usage==INPUT) 
    return (sepwarn(NOT_MET,"tag:%s  can't create a dff file for an input file\n",sep3dname));

  if(info->dff !=SEPNULL)
    return (sepwarn(NOT_MET,"tag:%s  can't section a sectioned tag\n",sep3dname));

   if(axis < 0 || axis > info->ndims)
     return (sepwarn(NOT_MET,"tag:%s illegal sectioned axis (%d) \n",sep3dname,axis));

     if(info->n[axis-1] < nsect)
       return (sepwarn(NOT_MET,
       "tag:%s sectioned axis (%d)  has less elements (%d) then sections(%d)\n",
       sep3dname,axis,axis,info->n[axis-1],nsect));


   /*create the dff tag*/
   info_dff=tag_info_sepdff(dff_base, info->usage);
   if(info_dff==SEPNULL )
     return(sepwarn(NOT_MET,"trouble creating dff tag %s \n",dff_base));
   if(info_dff->usage!=info->usage )
     return(sepwarn(NOT_MET,"dff tag %s already created with different usage\n",dff_base));
    if(0!=SEPDFF_set_mpi_info(info_dff))
    return(sepwarn(NOT_MET,"trouble setting mpi_info for tag=%s  \n",info_dff->name));

   if(0==strcmp(pattern,"BLOCK")) info_dff->pattern=BLOCK;
   else if( 0==strcmp(pattern,"SPREAD")) info_dff->pattern=SPREAD;
   else return (sepwarn(NOT_MET,"tag:%s unknown communication pattern  %s\n",
     sep3dname,pattern));



   if(0!=SEPDFF_set_nsect(info_dff,nsect))
     return (sepwarn(NOT_MET,"tag:%s trouble setting nsect \n", dff_base));

 
    if(0!=SEP3D_grab_axis(info,axis,&n,&o,&d,label,unit))
      return (sepwarn(NOT_MET,"tag:%s trouble grabbing axis info \n", sep3dname));

    if(0!=SEPDFF_set_sect_axis(info_dff,axis,info->n[axis-1]))
      return (sepwarn(NOT_MET,"tag:%s trouble setting sect axis  \n",dff_base));


  info_dff->master_data=master_data;
  if(info_dff->master_data==1) istart=0;
  else istart=1;


  machnames=(char**) malloc(sizeof(char*)*info_dff->nmpi);
  dff_path=(char**) malloc(sizeof(char*)*info_dff->nmpi);
  for(i=0; i < info_dff->nmpi; i++){
    machnames[i]=malloc(sizeof(char)*1024);
    dff_path[i]=malloc(sizeof(char)*1024);
  }

  if(0!=SEPDFF_grab_mpi_names(info_dff,machnames,dff_path)){
    for(i=0; i < info_dff->nmpi; i++) free(machnames[i]); free(machnames);
    for(i=0; i < info_dff->nmpi; i++) free(dff_path[i]); free(dff_path);
    return (sepwarn(NOT_MET,"tag:%s trouble getting mpi_names \n", sep3dname));
  }

  ns=(int*)malloc(sizeof(int)*nsect);
  sect_thread=(int*)malloc(sizeof(int)*nsect);
  axis_convert=(int*)malloc(sizeof(int)*info_dff->naxis_convert);
  last=0; impi=istart;
  nremain=info->n[axis-1];
 if(0!=sep3d_struct_init(info->name,"Combo232","SCRATCH"))
    return(sepwarn(NOT_MET,"trouble creating temportary tag \n"));

  for(isect=0; isect < info_dff->nsect; isect++){
    ns[isect]=ceil((double)nremain/((double)(nsect-isect)));
    nremain=nremain-ns[isect];
   if(0!= SEPDFF_make_dff_name(info_dff,temp2_ch,dff_path[impi],dff_base,isect,impi))
     return(sepwarn(NOT_MET,"trouble with dff name \n"));
    if(0!=SEPDFF_set_tag_sect(info_dff,isect,temp2_ch)
      || 0!=SEPDFF_set_mach_sect(info_dff,isect,machnames[impi])){
      free(axis_convert);
      for(i=0; i < info_dff->nmpi; i++) free(machnames[i]); free(machnames);free(sect_thread);
      for(i=0; i < info_dff->nmpi; i++) free(dff_path[i]); free(dff_path); free(ns);
      return (sepwarn(NOT_MET,"tag:%s trouble getting mpi_names \n", sep3dname));
    }
    SEPDFF_sect_name(info_dff,isect,temp_name);
    if(0!=sep3d_struct_init("Combo232",temp_name,"SCRATCH")){
      for(i=0; i < info_dff->nmpi; i++) free(machnames[i]); free(machnames);free(sect_thread);
      for(i=0; i < info_dff->nmpi; i++) free(dff_path[i]); free(dff_path);
      free(ns);
      free(axis_convert);
      return (sepwarn(NOT_MET,"tag:%s trouble initializing temp strucutre \n", sep3dname));
    }
    if(0==strcmp(pattern,"SPREAD")){
        if(0!=sep3d_set_axis(temp_name,axis,ns[isect],o+isect*d,
           d*nsect,label,unit)){
          for(i=0; i < info_dff->nmpi; i++) free(machnames[i]); free(machnames);
          free(sect_thread);
          for(i=0; i < info_dff->nmpi; i++) free(dff_path[i]); free(dff_path); free(ns);
           free(axis_convert);
          return (sepwarn(NOT_MET,"tag:%s trouble setting axis for section %d \n",
           sep3dname,isect));
        }
      for(i=0; i < ns[isect]; i++) axis_convert[isect+i*nsect]=isect;
    }
    if(0==strcmp(pattern,"BLOCK")){
        if(0!=sep3d_set_axis(temp_name,axis,ns[isect],o+last*d,
           d,label,unit)){
          for(i=0; i < info_dff->nmpi; i++) free(machnames[i]); free(machnames);
          free(sect_thread);
          for(i=0; i < info_dff->nmpi; i++) free(dff_path[i]); free(dff_path); free(ns);
          free(axis_convert);
          return (sepwarn(NOT_MET,"tag:%s trouble setting axis \n", sep3dname));
        }
      for(i=0; i < ns[isect]; i++) axis_convert[last+i]=isect;
    }
    last+=ns[isect];
    sect_thread[isect]=impi; 
    impi++;
    if(impi==info_dff->nmpi) impi=istart;
  }

  if(0!=SEPDFF_set_axis_convert(info_dff,axis_convert) ||
   0!=SEPDFF_set_sect_threads(info_dff,sect_thread)){
     for(i=0; i < info_dff->nmpi; i++) free(machnames[i]); free(machnames);
      free(sect_thread);
      for(i=0; i < info_dff->nmpi; i++) free(dff_path[i]); free(dff_path); free(ns);
      free(axis_convert);
      return(sepwarn(NOT_MET,
       "trouble setting axis convert and sect_thread info tag=%s \n",sep3dname));
  }
  free(axis_convert);
   info->dff=info_dff;

  for(i=0; i < info_dff->nmpi; i++) free(machnames[i]); free(machnames); free(sect_thread);
  for(i=0; i < info_dff->nmpi; i++) free(dff_path[i]); free(dff_path); free(ns);


 if(0!=SEPDFF_set_axis_thread_info(info_dff))
   return(sepwarn(NOT_MET,"trouble reading setting axis thread info tag=%s \n",sep3dname));

  if(0!=SEPDFF_open_local_sections(info_dff,1))
    return(sepwarn(NOT_MET,"trouble opening local sections \n"));
  sep3d_delete("Combo232");
  return(SUCCESS);
}

_XFUNCPROTOBEGIN
int sep3d_init_section_struct(char *sep3din, char *dffname, char *sep3dout,int master_data)
_XFUNCPROTOEND
{
sep_3d *info_in,*info_out;
sep_dff *info_dff_in,*info_dff_out;
char temp_ch2[1024],temp2_ch[1024],in_name[2049],out_name[2049];
char **machnames,**dff_path;
int i,isect,impi,istart;

  info_in = tag_info_sep3d(sep3din, INQUIRE);  /* get info on this tag */
  if(info_in == SEPNULL)
    return (sepwarn(INVALID_STRUC,"tag:%s  invalid struc\n",sep3din));
  info_out = tag_info_sep3d(sep3dout, INQUIRE);  /* get info on this tag */
  if(info_out == SEPNULL)
    return (sepwarn(INVALID_STRUC,"tag:%s  invalid struc\n",sep3dout));

  if(info_in->dff==SEPNULL) return(SUCCESS);

  info_dff_in=info_in->dff;
  info_dff_out=tag_info_sepdff(dffname, info_out->usage);
  if(info_dff_out==SEPNULL )
    return(sepwarn(NOT_MET,"trouble creating dff tag %s \n",dffname));
  if(0!=SEPDFF_set_mpi_info(info_dff_out))
    return(sepwarn(NOT_MET,"trouble setting mpi_info for tag=%s  \n",info_dff_out->name));

  /*start by making a true duplicate */
  SEPDFF_copy(info_dff_in,info_dff_out);

   info_dff_out->master_data=master_data;
   if(info_dff_out->master_data==1) istart=0;
   else istart=1;


  machnames=(char**) malloc(sizeof(char*)*info_dff_in->nmpi);
  dff_path=(char**) malloc(sizeof(char*)*info_dff_out->nmpi);
  for(i=0; i < info_dff_out->nmpi; i++){
    machnames[i]=malloc(sizeof(char)*1024);
    dff_path[i]=malloc(sizeof(char)*1024);
  }

  if(0!=SEPDFF_grab_mpi_names(info_dff_out,machnames,dff_path)){
    for(i=0; i < info_dff_out->nmpi; i++) free(machnames[i]); free(machnames);
    for(i=0; i < info_dff_out->nmpi; i++) free(dff_path[i]); free(dff_path);
    return (sepwarn(NOT_MET,"tag:%s trouble getting mpi_names \n", info_dff_out->name));
  }
  for(i=0; i < info_dff_in->nmpi; i++) free(machnames[i]); free(machnames);
 
  /*now all we want to do is change the dff names*/
   impi=istart;
  for(i=0; i < info_dff_out->nsect; i++){
    if(0!=SEPDFF_make_dff_name(info_dff_out,temp2_ch,dff_path[impi],dffname,i,impi))
     return(sepwarn(NOT_MET,"trouble with dff name \n"));
    if(0!=SEPDFF_set_tag_sect(info_dff_out,i,temp2_ch)){
      for(i=0; i < info_dff_out->nmpi; i++) free(dff_path[i]); free(dff_path);
      return(sepwarn(NOT_MET,"trouble setting tag_sect for %s \n",dffname));
    }
      SEPDFF_sect_name(info_dff_in,i,in_name);
      SEPDFF_sect_name(info_dff_out,i,out_name);
    if(0!=sep3d_grab_usage(in_name,temp2_ch))
      return(sepwarn(NOT_MET,"trouble grabbing usage for tag %s \n",info_dff_in->tag_sect[i]));
    if(0!=sep3d_struct_init(in_name,out_name,temp2_ch))
      return(sepwarn(NOT_MET,"trouble creating temportary tag \n"));

   impi++;
   if(impi==info_dff_out->nmpi) impi=istart;
  }
  for(i=0; i < info_dff_in->nmpi; i++) free(dff_path[i]); free(dff_path);
  info_out->dff=info_dff_out;
  if(0!=SEPDFF_open_local_sections(info_dff_out,1))
    return(sepwarn(NOT_MET,"trouble opening local sections \n"));
  return(SUCCESS);
}


_XFUNCPROTOBEGIN
int sep3d_init_section_tag(char *tag, char *sep3dname,int master_data)
_XFUNCPROTOEND
{
  sep_3d *info,*info_temp;
  sep_dff *info_dff,*info_dff2;
  char dff[512];
  char dff2[512];
  char temp_ch[512],tag_sect[512],mach_sect[512],tag_name[2049];
  char macnname[512];
  int ierr,nsect,i,axis,nold_num,nnew_num,*ncount_old,*ncount_new,localize_tag;
  int nmove,*move_list,*thread_num;
  char **machs_old,**tags_old,**dff_new;
  char **machs_new;
  int *reassign,nassign,ires;
   int nsz,*ns,ith;

  info = tag_info_sep3d(sep3dname, INQUIRE);  /* get info on this tag */
  if(info == SEPNULL)
    return (sepwarn(INVALID_STRUC,"tag:%s  invalid struc\n",sep3dname));

  if(1!=auxpar("dff","s",dff,tag)) return(SUCCESS);


  if(0==strcmp(dff,"-1")) return(SUCCESS);

  /*read in old dff structure*/
  info_dff=tag_info_sepdff(dff, info->usage);
  if(info_dff==SEPNULL )
    return(sepwarn(NOT_MET,"trouble creating dff tag %s \n",info_dff->name));
  if(info_dff->usage!=info->usage )
    return(sepwarn(NOT_MET,"dff tag %s already created with different usage\n",info_dff->name));

  if(0!=SEPDFF_set_mpi_info(info_dff))
  return(sepwarn(NOT_MET,"trouble setting mpi_info for tag=%s  \n",info_dff->name));

                                                                                              
  if(0!=SEPDFF_from_dff(dff,info_dff))
    return(sepwarn(NOT_MET,"trouble reading in dff structure for tag %s  dff=%s \n",
     tag,dff));

  sprintf(dff2,"%s",tag);
  info_dff2=tag_info_sepdff(dff2, SCRATCH);

  if(0!=SEPDFF_set_mpi_info(info_dff2))
  return(sepwarn(NOT_MET,"trouble setting mpi_info for tag=%s  \n",info_dff->name));
  if(info_dff2==SEPNULL )
    return(sepwarn(NOT_MET,"trouble creating dff tag %s \n",info_dff->name));


  if(0!=SEPDFF_copy(info_dff,info_dff2))
    return(sepwarn(NOT_MET,"trouble copying dff \n"));

  info_dff2->master_data=master_data;

  sprintf(temp_ch,"%s.reassign");
  if(getch(temp_ch,"d",&ires)==0) ires=1;
  
  if(ires==1){
fprintf(stderr,"IN REASSIGN \n");
    reassign=(int*) malloc(sizeof(int)*info_dff->nsect);
    if(0!=SEPDFF_assign_best(info_dff2,reassign,&nassign)){
      free(reassign); return(sepwarn(NOT_MET,"trouble assigning dff=%s tag=%s \n",
        dff2,tag));
    }
    free(reassign);
  }

  if(0!=sep3d_grab_usage(info->name,temp_ch))
     return(sepwarn(NOT_MET,"trouble grabbing usage for tag %s \n",info->name));
   
  /* this should be done differently so that each individual node opens its own section
    then passes the info*/
  for(i=0; i< info_dff2->nsect; i++){
    SEPDFF_sect_name(info_dff2,i,tag_name);
    if(info_dff2->sect_thread[i]==info_dff2->impi)
      auxinout(info_dff2->tag_sect[i]);
    if(0!=sep3d_tag_init(info_dff2->tag_sect[i],tag_name,temp_ch))
      return(sepwarn(NOT_MET,"trouble initializing tag tag %s \n",info_dff2->tag_sect[i]));
  }



  info->dff=info_dff2;
  
  if(info_dff->sect_axis==info->ndims){

    ns=(int*)malloc(sizeof(int)*info->ndims);

    for(ith=0; ith < info_dff->nsect; ith++){
       SEPDFF_sect_name(info_dff2,ith,tag_name);
       info_temp=tag_info_sep3d(tag_name,INQUIRE);
       if(info_temp==SEPNULL)
         return(sepwarn(NOT_MET,"internal error: %s not initialized tag=%s sect=%d\n",
          tag_name,tag,ith));
       
       if(info_temp->ndims!=info->ndims){
         for(i=0; i < info_temp->ndims; i++) ns[i]=i+1;
         for(i=info_temp->ndims; i < info->ndims; i++) ns[i]=info_temp->ndims;
    
         if(0!=sep3d_change_dims(info_temp->name,info->ndims,ns)){
          free(ns);
          return(sepwarn(NOT_MET,"Trouble reshaping tag=%s \n",
           info_temp->name));
         }
         if(0!=sep3d_reed_axis(info_dff->tag_sect[ith],info_dff->sect_axis,
         &info_temp->n[info->ndims-1], &info_temp->o[info->ndims-1], 
         &info_temp->d[info->ndims-1], info_temp->label[info->ndims-1], 
         info_temp->unit[info->ndims-1]))
           return(sepwarn(NOT_MET,"trouble reading axis  \n"));
       }
    }
    free(ns);
  }

  return(SUCCESS);
}

_XFUNCPROTOBEGIN
int sep3d_desection_tag(char *sep3dname)
_XFUNCPROTOEND
{
 sep_3d *info,*info2;
  int nbuf;
  char *buf1,*buf2; 
                                                                                              
  info = tag_info_sep3d(sep3dname, INQUIRE);  /* get info on this tag */
  if(info == SEPNULL)
    return (sepwarn(INVALID_STRUC,"tag:%s  invalid struc\n",sep3dname));


  if(info->dff==SEPNULL) return(SUCCESS);
  SEPDFF_delete(info->dff);
  info->dff=SEPNULL;
  return(SUCCESS);
}



_XFUNCPROTOBEGIN
int  SEPDFF_make_dff_name(sep_dff *info,char *temp_ch,char *dff_path, char *name, int sect,int impi)
_XFUNCPROTOEND
{
 char *front;
if(impi==info->impi){
  front = strrchr(name,'/');
  if(front == 0 ) front = name; else front++;
  sprintf(temp_ch,"%s%s%d_XXXXXX",dff_path,front,sect);
  mkstemp(temp_ch);
}
if(0!=sep3d_broadcast_string(temp_ch,impi))
  return(sepwarn(NOT_MET,"trouble broadcasting name \n"));

return(SUCCESS);
}

_XFUNCPROTOBEGIN
int  sep3d_open_local_sections(char *tag,int clean)
_XFUNCPROTOEND
{
 sep_3d *info;

  info = tag_info_sep3d(tag, INQUIRE);  /* get info on this tag */
  if(info == SEPNULL)
    return (sepwarn(INVALID_STRUC,"tag:%s  invalid struc\n",tag));
  if(info->dff != SEPNULL){
    return(SEPDFF_open_local_sections(info->dff,clean));
  }
  else return(0);
}


_XFUNCPROTOBEGIN
int  SEPDFF_open_local_sections(sep_dff *dff,int clean)
_XFUNCPROTOEND
{
int i;

for(i=0; i<dff->nsect_ithread; i++){
  if(clean==1){
    auxout(dff->tag_sect[dff->sect_ithread[i]]);
    auxclose(dff->tag_sect[dff->sect_ithread[i]]);
  }
  auxinout(dff->tag_sect[dff->sect_ithread[i]]);
}



return(0);

}
