#define SET_SDOC 1
#include<sitedef.h>
#include<superset.h>
#include<superset_internal.h>

_XFUNCPROTOBEGIN
int SEP3D_read_data_section(sep_3d *info, sep_dff *info_dff, int ithread,
 char *data, int isect)
_XFUNCPROTOEND
{
int esize,nelem,nh;

if(info_dff->impi == info_dff->sect_thread[isect]){
  if(0!=sep3d_read_data(  info_dff->tag_sect[isect],info->name,
   info->nwind[0],info->fwind[0],info->jwind[0],data))
    return(sepwarn(NOT_MET,"trouble reading section data tag=%s section=%d\n",       info_dff->tag_sect[isect],isect));
}




if(info_dff->impi == info_dff->sect_thread[isect] || info_dff->impi==ithread){
  esize=SEP3D_get_esize(info);
  nelem=info->ncoord*info->n[0];

  if(0!=sep3d_pass_data(info_dff->impi,
   info_dff->sect_thread[isect],ithread,nelem,esize,data)) 
    return(sepwarn(NOT_MET,
   "tag=%s trouble passing section %d from %d to %d \n",
    info->name,isect,ithread,info_dff->sect_thread[isect]));
}
return(SUCCESS);
}
_XFUNCPROTOBEGIN
int SEP3D_read_header_section(sep_3d *info, sep_dff *info_dff, int ithread,
 int isect)
_XFUNCPROTOEND
{
int esize,nelem,nh;

if(info_dff->impi == info_dff->sect_thread[isect]){
  if(0!=sep3d_read_headers(  info_dff->tag_sect[isect],info->name,
   &info->nwind[1],&info->fwind[1],&info->jwind[1],&nh))
    return(sepwarn(NOT_MET,"trouble reading section header tag=%s section=%d\n",       info_dff->tag_sect[isect],isect));
}
if(info_dff->impi == info_dff->sect_thread[isect] || info_dff->impi==ithread){
  if(0!=sep3d_pass_headers(info_dff->impi,info->name,
   info_dff->sect_thread[isect],ithread)) return(sepwarn(NOT_MET,
   "tag=%s trouble passing section %d from %d to %d \n",
    info->name,isect,ithread,info_dff->sect_thread[isect]));
}
return(SUCCESS);
}

_XFUNCPROTOBEGIN
int SEP3D_read_data_section_list(sep_3d *info, sep_dff *info_dff, char *data,
   int nsect, int *sect_list)
_XFUNCPROTOEND
{
int isect,n123,i,nh,nsz,idim;
int nout,fout,jout,ncount;
int nt,ft,jt;
char *temp_data;
int *sect_index,*index;
sep_3d *info_temp;
sep_3d *info_small;

    if(info->file_format==REGULAR){
      if(0!=SEP3D_wind_coords(info,&nsz))
         return(sepwarn(NOT_MET,
       "trouble grabbing coords from window parameters tag=%s \n",info->name));
       if(nsz==0) return(SUCCESS);
     }
     else nsz=info->ncoord; 
   if(nsz==0) return(SUCCESS);
                                                                                
                                                                                
  index=(int*)malloc(sizeof(int)*nsz);
  if(0!=SEP3D_find_sect_index(info,info,info_dff,index)){
    free(index);
    return(sepwarn(NOT_MET,"trouble finding sect index map for tag=%s \n",info->name));
  }
  


  if(nsect==1) temp_data=data;
  for(i=0,ncount=0; i < nsect; i++){
    isect=sect_list[i];
    info_small= tag_info_section(info_dff,isect, INQUIRE);

    if(0!=SEP3D_localize_window(info,info_small,isect)){
      free(index);
      return(sepwarn(NOT_MET,"trouble localizing window tag=%s sect=%d \n",
        info->name,isect));
    }
    if(info->file_format==REGULAR){
       for(idim=1,nsz=1; idim < info->ndims; idim++) 
          nsz=nsz*info_small->nwind[idim];
    }
    else nsz=info_dff->wind_sect_nh[isect];

    


    if(nsect>1)
       temp_data=(char*)malloc(sizeof(char)*nsz*
      SEP3D_get_esize(info_small)*info->nwind[0]);


    if(0!=SEP3D_read_data_section(info_small,info_dff,
      info_dff->sect_thread[isect],temp_data,isect)){
        free(index); free(temp_data);
        return(sepwarn(NOT_MET,"tag=%s section=%d trouble reading seciton\n",
         info->name,isect));
      }


   if(nsect>1){
    if(0!=SEP3D_add_data_section(info_small,temp_data,info,info_dff,data,isect,
       info->ncoord,index)){
         free(index);free(temp_data);
         return(sepwarn(NOT_MET,
        "trouble adding section %d to tag=%s \n",isect,info->name));
    }
   }

    if(nsect>1) free(temp_data);
    ncount+=info_small->ncoord;

  }
  free(index);

  /* could delete as we go*/
  return(SUCCESS);
}

#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int sep3d_read_header_local(char *sep3din,int *nwind, int *fwind, int *jwind, int *nh)
_XFUNCPROTOEND
#endif
{
sep_3d *info;
sep_dff *info_dff;
int ierr;
int i,nsz;
int nt[10],ft[10],jt[10];
int n[2]; 
info = tag_info_sep3d(sep3din, INQUIRE);  /* get info on this tag */
if(info == SEPNULL)
  return (sepwarn(INVALID_STRUC,"tag:%s  invalid struc\n",sep3din));

info_dff=info->dff;
if(info_dff == SEPNULL)
  return (sepwarn(INVALID_STRUC,"tag:%s  not a distributed tag\n",info->name));


for(i=1; i < info->ndims;i++){
info->nwind[i]=nwind[i-1];
info->fwind[i]=fwind[i-1];
info->jwind[i]=jwind[i-1];
}
  if(info->file_format==REGULAR){
    info->fwind[0]=0;
    info->jwind[0]=1;
    info->nwind[0]=info->n[0];
     if(0!= sep3d_buf_size(info->name,info->nwind,info->fwind,info->jwind,n)){
       free(n);
       return(sepwarn(NOT_MET,"trouble calculating buffer size tag=%s \n",info->name));
     } 
     *nh=n[1];
      return(SUCCESS);
  }

ierr=SEP3D_read_header_section_list(info, info_dff,
  info_dff->nsect_ithread,info_dff->sect_ithread);


*nh=info->ncoord;
  return(ierr);
}




#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int sep3d_read_data_local(char *sep3din,int nwind, int fwind, int jwind, char *data)
_XFUNCPROTOEND
#endif
{
sep_3d *info;
sep_dff *info_dff;
int ierr;
info = tag_info_sep3d(sep3din, INQUIRE);  /* get info on this tag */
if(info == SEPNULL)
  return (sepwarn(INVALID_STRUC,"tag:%s  invalid struc\n",sep3din));

info_dff=info->dff;
if(info_dff == SEPNULL)
  return (sepwarn(INVALID_STRUC,"tag:%s  not a distributed tag\n",info->name));


info->nwind[0]=nwind;
info->fwind[0]=fwind;
info->jwind[0]=jwind;


ierr=SEP3D_read_data_section_list(info, info_dff,data,
  info_dff->nsect_ithread,info_dff->sect_ithread);
  return(ierr);
}






_XFUNCPROTOBEGIN
int SEP3D_read_header_section_list(sep_3d *info, sep_dff *info_dff,
   int nsect, int *sect_list)
_XFUNCPROTOEND
{
int isect,n123,i,nh,nsz,idim,ia;
int nout,fout,jout,ncount;
int nt,ft,jt,iloc;
char *temp_data;
int *sect_index,*index;
sep_3d *info_temp;
sep_3d *info_small;

/*  if(info->ncoord==0) return(SUCCESS);*/
                                                                                
                                                                                
  for(i=0,ncount=0; i < nsect; i++){
    isect=sect_list[i];
    info_small= tag_info_section(info_dff,isect, INQUIRE);


    if(0!=SEP3D_window_section_2(info,info_small,isect)){
      free(index);
      return(sepwarn(NOT_MET,"trouble localizing window tag=%s sect=%d \n",
        info->name,isect));
    }
    if(0!=SEP3D_read_header_section(info_small,info_dff,
      info_dff->sect_thread[isect],isect)){
        free(index);
        return(sepwarn(NOT_MET,"tag=%s section=%d trouble reading seciton\n",
         info->name,isect));
      }
     info_dff->wind_sect_nh[isect]=info_small->ncoord;
     ncount+=info_small->ncoord; 
   }


  if(0!=SEP3D_alloc_coord(info,ncount)) return(sepwarn(NOT_MET,
  "trouble allocating coords tag=%s \n",info->name));

  if(0!=sep3d_set_nh(info->name,ncount)) return(sepwarn(NOT_MET,
  "trouble allocating headers tag=%s \n",info->name));

  index=(int*)malloc(sizeof(int)*ncount);
  for(i=0,iloc=0; i < nsect; i++){
    isect=sect_list[i];
    for(ia=0; ia < info_dff->wind_sect_nh[isect]; ia++,iloc++) index[iloc]=isect;
  }
  

  for(i=0,ncount=0; i < nsect; i++){
    isect=sect_list[i];
    info_small= tag_info_section(info_dff,isect, INQUIRE);
                                                                                
    if(0!=SEP3D_add_header_section(info_small,info_dff,info,
       isect,info->ncoord,index)) {
        free(index);
        return(sepwarn(NOT_MET,
        "trouble adding section %d to tag=%s \n",isect,info->name));
    }

    ncount+=info_dff->wind_sect_nh[isect];
  }

   free(index);
  /* could delete as we go*/
  return(SUCCESS);
}

