/*
* This routine draws and annotates an axis. Any orientation.
* Axis goes from (x1,y1) to (x2,y2). Tic marks are perpendicular
* to it, on the right side as you go from point 1 to point 2.
*
* Author - S. Cole (SEP)   11 Dec 92
*/
#include "glplot.h"

#ifdef USE_PROTO
int gl_simpleaxis ( float x1, float y1, float x2, float y2, float num1, float num2,
  float dnum, float dnumticin, float ltic, char* label, float labelsz )
#else
int gl_simpleaxis ( x1, y1, x2, y2, num1, num2, dnum, dnumticin, ltic, label, labelsz )
float x1, x2, y1, y2, num1, num2, dnum, dnumticin, ltic;
char *label;
float labelsz;
#endif
{
float		ch, xpath, ypath, xup, yup;
char            string[10];
float		dist, costh, sinth, dtic, xpos, ypos;
float		dxtic, dytic, loc, num;
float		pad;
float		dnumtic;

    /* pad is the space skipped between tic and number, and number and label */
    pad = 0.15;
    ch = labelsz / 33.;

    gl_color (7);
    gl_move(x1,y1);
    gl_draw(x2,y2);

    /* compute sines and cosines of axis angle */
    dist = sqrt((x2-x1)*(x2-x1) + (y2-y1)*(y2-y1));
    costh = (x2 - x1)/dist;
    sinth = (y2 - y1)/dist;
    /* x and y sizes of tic marks */
    dxtic = ltic * sinth;
    dytic = -ltic * costh;

    /* 
     * Figure out which quadrant we are in.
     * This determines which side of the line the tic
     * marks will go on, and how the text will be aligned.
     * For each quadrant we compute the vplot text path
     * and up vectors.
     */
    if (x1 <= x2 && y1 <= y2) {
        vp_tjust(TH_CENTER,TV_TOP);
        xpath = ch * costh;
        ypath = ch * sinth;
        xup = -ch * sinth;
        yup = ch * costh;
    } else if (x1 > x2 && y1 <= y2) {	
        vp_tjust(TH_CENTER,TV_BOTTOM);
        xpath = -ch * costh;
        ypath = -ch * sinth;
        xup = ch * sinth;
        yup = -ch * costh;
    } else if (x1 > x2 && y1 > y2) {	
        vp_tjust(TH_CENTER,TV_BOTTOM);
        xpath = -ch * costh;
        ypath = -ch * sinth;
        xup = ch * sinth;
        yup = -ch * costh;
    } else if (x1 <= x2 && y1 > y2) {	
        vp_tjust(TH_CENTER,TV_TOP);
        xpath = ch * costh;
        ypath = ch * sinth;
        xup = -ch * sinth;
        yup = ch * costh;
    }

    /* call gl_opttic to find the optimum tic mark interval */
    dnumtic = dnumticin;
    if (dnumtic == 0.) gl_opttic(num1,num2,dist,num1,&dnumtic,labelsz);
    if (num1 > num2) dnumtic *= -1.;

    /* figure out the tic mark spacing */
    dtic = dnumtic/(num2-num1) * dist; 
    if (dtic < 0.) dtic *= -1.;

    /*
     * move to each tic mark location, draw the tic and the number
     */
    for (loc=0., num=num1; loc < dist; loc+=dtic, num+=dnumtic) {
        sprintf(string,"%1.5g",num);
	xpos = x1 + loc * costh;
	ypos = y1 + loc * sinth;
        vp_move(xpos,ypos);
	vp_draw(xpos+dxtic,ypos+dytic);
	vp_gtext(xpos+dxtic+pad*sinth,ypos+dytic-pad*costh,xpath,ypath,xup,yup,string);
    }
    /* now the axis label */
    xpos = x1 + loc/2. * costh + dxtic + ch * sinth + 2. * pad * sinth;
    ypos = y1 + loc/2. * sinth + dytic - ch * costh - 2. * pad * costh;
    vp_gtext(xpos,ypos,xpath,ypath,xup*1.5,yup*1.5,label);

	return 0;
}
