module wei_wavespace_mod
use wei_basics_mod
implicit none
type(sep3d),private, save :: W
real, private,pointer, save :: omega(:),dstep(:)
integer,private, save :: opsign
logical,private, save :: adj,add


contains

logical function init_reg_wave_space(win)
type(sep3d) :: win

init_reg_wave_space=.false.

call init_sep3d(win,"SCRATCH","COMPLEX","REGULAR",(/1,1,1,1,1,1,1,1/))
call from_param("amx_n",win%n(1));call from_param("amy_n",win%n(2))
call from_param("ahx_n",win%n(3));call from_param("ahy_n",win%n(4))
call from_param("az__n",win%n(5));call from_param("aw__n",win%n(6))

call from_param("amx_o",win%o(1));call from_param("amy_o",win%o(2))
call from_param("ahx_o",win%o(3));call from_param("ahy_o",win%o(4))
call from_param("az__o",win%o(5));call from_param("aw__o",win%o(6))

call from_param("amx_d",win%d(1));call from_param("amy_d",win%d(2))
call from_param("ahx_d",win%d(3));call from_param("ahy_d",win%d(4))
call from_param("az__d",win%d(5));call from_param("aw__d",win%d(6))
call sep3d_set_sep3d(win)

allocate(omega(my_nomp()))
allocate(dstep(my_nomp()))

call from_param("nzb",win%n(7),1);call from_param("nwb",win%n(8),1)

if(win%n(8) > win%n(6) .or. win%n(8) <1) then
  if(sep_thread_num()==0) write(0,*) "illegal nwb value",win%n(6)
  return
end if

if(win%n(7) > win%n(6) .or. win%n(5) <1) then
  if(sep_thread_num()==0) write(0,*) "illegal nzb value",win%n(5)
  return
end if

init_reg_wave_space=.true.

end function

logical function check_space_logic(wsep,dsep,rsep,ssep)
type(sep3d) :: wsep,dsep,rsep,ssep(:)
integer :: i
check_space_logic=.false.

write(0,*) "aaw"
   if(.not. axis_check_param_file("m_x",1,1,wsep,dsep)) then
    if(sep_thread_num()==0) write(0,*)  "data and wave space inconsistent"
    return
   end if
write(0,*) "aax"
   if(.not. axis_check_param_file("m_y",2,2,wsep,dsep)) then
    if(sep_thread_num()==0) write(0,*)  "data and wave space inconsistent"
    return
   end if
write(0,*) "aay"
   if(.not. axis_check_param_file("h_x",3,3,wsep,dsep)) then
    if(sep_thread_num()==0) write(0,*)  "data and wave space inconsistent"
    return
   end if
write(0,*) "aaz"
   if(.not. axis_check_param_file("h_y",4,4,wsep,dsep)) then
    if(sep_thread_num()==0) write(0,*)  "data and wave space inconsistent"
    return
   end if
write(0,*) "aa0"
   if(.not. axis_check_param_file("m_x",1,1,wsep,rsep)) then
    if(sep_thread_num()==0) write(0,*)  "image and wave space inconsistent"
    return
   end if
   if(.not. axis_check_param_file("m_y",2,2,wsep,rsep)) then
    if(sep_thread_num()==0) write(0,*)  "image and wave space inconsistent"
    return
   end if
   if(.not. axis_check_param_file("h_x",3,3,wsep,rsep)) then
    if(sep_thread_num()==0) write(0,*)  "image and wave space inconsistent"
    return
   end if
   if(.not. axis_check_param_file("h_y",4,4,wsep,rsep)) then
    if(sep_thread_num()==0) write(0,*)  "image and wave space inconsistent"
    return
   end if
   if(.not. axis_check_param_file("z",5,5,wsep,rsep)) then
    if(sep_thread_num()==0) write(0,*)  "image and wave space inconsistent"
    return
   end if
   if(.not. axis_check_param_file("w",5,6,dsep,wsep)) then
    if(sep_thread_num()==0) write(0,*)  "data and wave space inconsistent"
    return
   end if
   check_space_logic=.true.
end function

   logical function axis_check_param_file(axisdes,iax1,iax2,psep,fsep)
    type(sep3d) :: psep,fsep
    character(len=*) axisdes
    logical :: stat
    integer :: iax1,iax2
    logical  :: exact
    real   :: mye
    mye=epsilon(mye)*100.
    axis_check_param_file=.true.
                                                                                           
    if(psep%n(iax1) .ne. fsep%n(iax2)) then
      if(sep_thread_num()==0) write(0,("a,i2,a,a,a,i3,a,a,a,i3,a")) "n of axis",iax1,&
        " of ",trim(fsep%tag),"(",fsep%n(iax2),&
      ") and ",trim(axisdes),"(",psep%n(iax1),") are inconsistent"
      axis_check_param_file=.false.
    end if
    if(psep%n(iax1) >1) then
      if(abs(psep%o(iax1) - fsep%o(iax2)) > mye) then
         if(sep_thread_num()==0) write(0,("a,i2,a,a,a,f12.5,a,a,a,f12.5,a")) "o of axis",&
           iax1," of ",trim(fsep%tag),"(",fsep%o(iax2),&
        ") and ",trim(axisdes),"(",psep%o(iax1),") are inconsistent"
        axis_check_param_file=.false.
       end if
      if(abs(psep%d(iax1) - fsep%d(iax2)) > mye) then
         if(sep_thread_num()==0) write(0,("a,i2,a,a,a,f12.5,a,a,a,f12.5,a")) "d of axis",&
          iax1," of ",trim(fsep%tag),"(",fsep%d(iax2),&
        ") and ",trim(axisdes),"(",psep%d(iax1),") are inconsistent"
         axis_check_param_file=.false.
       end if
    end if
   end function
!
! Grab and store
!

subroutine store_current_dstep(d)
real :: d
dstep(my_iomp())=d
end subroutine

subroutine grab_current_dstep(d)
real :: d
d=dstep(my_iomp())
end subroutine



subroutine grab_current_opsign(o)
integer :: o
o=opsign
end subroutine

subroutine store_current_omega(o)
real :: o
omega(my_iomp())=o
end subroutine

subroutine grab_current_omega(o)
real :: o
o=omega(my_iomp())*2.*pi
end subroutine

subroutine store_current_add(a)
logical :: a
add=a
end subroutine

subroutine grab_current_add(a)
logical :: a
a=add
end subroutine


subroutine store_current_adj(a)
logical :: a
adj=a
opsign = goB2T
if(adj) opsign = goT2B
end subroutine

subroutine grab_current_adj(a)
logical :: a
a=adj
end subroutine


subroutine grab_current_wave_space(win)
type(sep3d) :: win
win=W
end subroutine


subroutine store_current_wave_space(win)
type(sep3d) :: win
W=win
end subroutine

                                                                                           

end module
