!$=head1 NAME
!$
!$ Envelope - calculate signal amplitude
!$
!$=head1 SYNOPSIS
!$
!$       <in.H  Envelope  [tc1=1 tc2=1 freq=0]  >out.H
!$
!$=head1 INPUT PARAMETERS
!$
!$=over 4
!$
!$=item  tc1   - integer
!$
!$       half width of smoothing triangle on 1-axis.
!$
!$=item tc2   - integer
!$
!$      half width of smoothing triangle on 2-axis.
!$      units of tc are sample points.
!$
!$=item freq  - integer
!$
!$      [1] produce instantaneous frequency instead of envelope
!$
!$=back
!$
!$=head1 DESCRIPTION
!$
!$ Analytic signal amplitude.
!$
!$=head1 CATEGORY
!$
!$B<seis/filter>
!$
!$=cut
!>
! KEYWORDS      envelope instantaneous frequency
!
!
program envelope 
  implicit none
  integer xn1, xn2, n1,n2,n3, pad2
  real,allocatable::         data(:),  top(:,:), bot(:,:)
  complex ,allocatable::cdata(:), ctop(:)
  integer hetch,putch
  call initpar()
call sep_add_doc_line("NAME")
call sep_add_doc_line("     Envelope - calculate signal amplitude")
call sep_add_doc_line("")
call sep_add_doc_line("SYNOPSIS")
call sep_add_doc_line("           <in.H  Envelope  [tc1=1 tc2=1 freq=0]  >out.H")
call sep_add_doc_line("")
call sep_add_doc_line("INPUT PARAMETERS")
call sep_add_doc_line("    tc1 - integer")
call sep_add_doc_line("               half width of smoothing triangle on 1-axis.")
call sep_add_doc_line("")
call sep_add_doc_line("    tc2 - integer")
call sep_add_doc_line("              half width of smoothing triangle on 2-axis.")
call sep_add_doc_line("              units of tc are sample points.")
call sep_add_doc_line("")
call sep_add_doc_line("    freq - integer")
call sep_add_doc_line("              [1] produce instantaneous frequency instead of envelope")
call sep_add_doc_line("")
call sep_add_doc_line("DESCRIPTION")
call sep_add_doc_line("     Analytic signal amplitude.")
call sep_add_doc_line("")
call sep_add_doc_line("CATEGORY")
call sep_add_doc_line("    seis/filter")
call sep_add_doc_line("")

  call doc('SOURCE')
    if (0>= hetch('n1','d',xn1 )) then
      call erexit('Trouble reading  n1 from history  ')
    end if
    if (0>= hetch('n2','d',xn2 )) then
      xn2 = 1
    end if
    if (0>= hetch('n3','d',n3 )) then
      n3 = 1
    end if
  n1 = pad2( xn1)
  n2 = pad2( xn2)
  allocate(data(n1),  top(n1,n2), bot(n1,n2))
  allocate(cdata(n1), ctop(n1))
  call goodie(xn1,xn2, n1,n2,n3, data, cdata, top,ctop, bot)
end program 
subroutine goodie( xn1,xn2, n1,n2,n3, data, cdata, top,ctop, bot)
  implicit none
  integer xn1, xn2, i1,n1,i2,n2, i3,n3, tc1, tc2, freq
  real     data(n1), top(n1,n2), bot(n1,n2)
  complex cdata(n1), ctop(n1)
  integer getch,putch
    if (0>= getch('tc1','d',tc1 )) then
      tc1 = 1
    end if
    if (0.ne.putch('Read  from param: #tc1 ','d',tc1)) then
      call erexit('Trouble writing tc1 to history')
    end if
    if (0>= getch('tc2','d',tc2 )) then
      tc2 = 1
    end if
    if (0.ne.putch('Read  from param: #tc2 ','d',tc2)) then
      call erexit('Trouble writing tc2 to history')
    end if
    if (0>= getch('freq','d',freq )) then
      freq = 0
    end if
    if (0.ne.putch('Read  from param: #freq ','d',freq)) then
      call erexit('Trouble writing freq to history')
    end if
  call hclose()        
  do i3=1,n3  
    do i2= 1, xn2  
      call sreed( 'in', data, 4*xn1 )
      do i1= 1, n1
        cdata(i1) = 0.
      end do 
      do i1= 1, xn1
        cdata(i1) = data(i1)
      end do 
      call ftu( 1., n1, cdata)
      cdata(1)      = cdata(1)      / 2.
      cdata(1+n1/2) = cdata(1+n1/2) / 2.
      do i1 = n1/2+2, n1
        cdata(i1) = 0.
      end do 
      do i1=1,n1
        ctop(i1) = cmplx(0., 2.*3.14159*(i1-1.)/n1) * cdata(i1)
      end do 
      call ftu( -1., n1, cdata)
      call ftu( -1., n1, ctop)
      do i1=1,n1  
        bot(i1,i2) = conjg( cdata(i1)) * cdata(i1)
        top(i1,i2) = conjg( cdata(i1)) * ctop (i1) / (0.,1.)
      end do
    end do 
    call triangle2( tc1, tc2, n1, n2, bot, bot)
    call triangle2( tc1, tc2, n1, n2, top, top)
    do i2=1,n2
      do i1=1,n1
        if ( bot(i1,i2) .ne. 0.) then
          top(i1,i2) = top(i1,i2) / bot(i1,i2)
        else
          top(i1,i2) = 0.
        end if
      end do
    end do 
    do i2=1,xn2  
      if ( freq .eq. 0 ) then
        do i1=1,n1
          data(i1) = sqrt( bot(i1,i2) )
        end do
      else
        do i1=1,n1
          data(i1) =  top(i1,i2)
        end do
      end if 
      call srite( 'out', data, 4*xn1)
    end do
  end do
end subroutine 
!subroutine ftu( signi, nx, cx ){
!integer nx, i, j, k, m, istep, pad2,a
!real  signi, scale, arg
!complex cx(nx), cmplx, cw, cdel, ct
!if( nx != pad2(nx) )  call erexit('ftu: nx not a power of 2')
!scale = 1. / sqrt( 1.*nx)
!do i= 1, nx
!  cx(i) = cx(i) * scale
!j = 1;  k = 1
!do i= 1, nx {
!  if (i<=j) { ct = cx(j); cx(j) = cx(i); cx(i) = ct }
!  m = nx/2
!  while (j>m && m>1) { j = j-m; m = m/2 }   # "&" means .AND.
!  j = j+m
!  }
!a=0
!while(a==0) {
!  istep = 2*k;   cw = 1.;   arg = signi*3.14159265/k
!  cdel = cmplx( cos(arg), sin(arg))
!  do m= 1, k {
!    do i= m, nx, istep
!      { ct=cw*cx(i+k);  cx(i+k)=cx(i)-ct;  cx(i)=cx(i)+ct }
!    cw = cw * cdel
!    }
!  k = istep
!  if(k>=nx) break
!  }
!}
  
