!$=head1 NAME
!$
!$Median -  median filter
!$
!$=head1 SYNOPSIS
!$
!$Median < in.H > out.H
!$
!$=head1 INPUT PARAMETERS
!$
!$=over 4
!$
!$=item wind1 - integer
!$
!$      [0]: median window distance in n1 direction
!$
!$=item wind2 - integer
!$
!$      [0]: median window distance in n2 direction
!$
!$=item wind3 - integer
!$
!$      [0]: median window distance in n3 direction
!$
!$=back
!$
!$=head1 DESCRIPTION
!$
!$ Median smoother.  Window size is (2*wind1+1)*(2*wind2+1)*(2*wind3+1)
!$
!$=head1 EXAMPLE
!$
!$                Median < in.H wind1=10 wind2=5 wind3=2 > out.H
!$                                ??????????????????????????????
!$
!$=head1 CATEGORY
!$
!$B<seis/filter>
!$
!$=cut
!$
!
!>
!-------------------------------------------------
!
!Author: Robert Clapp, ESMB 463, 7230253
!
!Date Created:Wed Apr 15 13:51:56 PDT 1998
!
!Purpose:
!
program median 
  use sorting_mod
  implicit none
  integer :: n1,n2,n3,wind1,wind2,wind3,ierr,panel,sz3,sz2,sz1
  integer :: i1,i2,i3,b1,b2,b3,e1,e2,e3,sreed,srite,center
  real,allocatable :: input(:,:,:),output(:,:),temp_v(:)
! get parameters
  integer getch,hetch,putch
  call initpar()
call sep_add_doc_line("NAME")
call sep_add_doc_line("    Median - median filter")
call sep_add_doc_line("")
call sep_add_doc_line("SYNOPSIS")
call sep_add_doc_line("    Median < in.H > out.H")
call sep_add_doc_line("")
call sep_add_doc_line("INPUT PARAMETERS")
call sep_add_doc_line("    wind1 - integer")
call sep_add_doc_line("              [0]: median window distance in n1 direction")
call sep_add_doc_line("")
call sep_add_doc_line("    wind2 - integer")
call sep_add_doc_line("              [0]: median window distance in n2 direction")
call sep_add_doc_line("")
call sep_add_doc_line("    wind3 - integer")
call sep_add_doc_line("              [0]: median window distance in n3 direction")
call sep_add_doc_line("")
call sep_add_doc_line("DESCRIPTION")
call sep_add_doc_line("     Median smoother.  Window size is (2*wind1+1)*(2*wind2+1)*(2*wind3+1)")
call sep_add_doc_line("")
call sep_add_doc_line("EXAMPLE")
call sep_add_doc_line("                    Median < in.H wind1=10 wind2=5 wind3=2 > out.H")
call sep_add_doc_line("                                    ??????????????????????????????")
call sep_add_doc_line("")
call sep_add_doc_line("CATEGORY")
call sep_add_doc_line("    seis/filter")
call sep_add_doc_line("")

  call doc('SOURCE')
    if (0>= hetch('n1','d',n1 )) then
      call erexit('Trouble reading  n1 from history  ')
    end if
    if (0>= hetch('n2','d',n2 )) then
      call erexit('Trouble reading  n2 from history  ')
    end if
    if (0>= hetch('n3','d',n3 )) then
      n3 = 1
    end if
    if (0>= getch('wind1','d',wind1 )) then
      call erexit('Trouble reading  wind1 from param  ')
    end if
    if (0.ne.putch('Read  from param: #wind1 ','d',wind1)) then
      call erexit('Trouble writing wind1 to history')
    end if
    if (0>= getch('wind2','d',wind2 )) then
      wind2 = 2
    end if
    if (0.ne.putch('Read  from param: #wind2 ','d',wind2)) then
      call erexit('Trouble writing wind2 to history')
    end if
    if (0>= getch('wind3','d',wind3 )) then
      wind3 = 0
    end if
    if (0.ne.putch('Read  from param: #wind3 ','d',wind3)) then
      call erexit('Trouble writing wind3 to history')
    end if
! allocate working space
  allocate(input(n1,n2,wind3*2+1),output(n1,n2),temp_v((2*wind1+1)*(2&
    &*wind2+1)*(2*wind3+1)),stat=ierr)
  if (ierr.ne.0) then
    call erexit("trouble allocating arrays")
  end if
! read input data
  if (n1*n2*(2*wind3+1)*4 .ne. sreed("in",input,n1*n2*(2*wind3+1)*4))&
    & then
    call erexit("trouble reading data")
  end if
! compute and apply median filter
  do i3=1,n3 
    if (i3>  wind3+1 .and. i3 <= n3-wind3) then
      write(0,*) "before read",i3
      input=cshift(input,1,3)
      if (n1*n2*4.ne.sreed("in",input(:,:,2*wind3+1),n1*n2*4)) then
        call erexit("trouble readin in data")
      end if
    end if
    if (i3<= wind3) then
      e3=i3+wind3
    else
      e3=wind3*2+1
    end if 
    if (i3> n3-wind3) then
      b3=wind3+1-(n3-i3)
    else
      b3=1
    end if 
    sz3=e3-b3+1
    write(0,*) i3,b3,e3,sz3
    do i2=1,n2 
      b2=max(1,i2-wind2)
      e2=min(n2,i2+wind2)
      sz2=e2-b2+1
      do i1=1,n1  
        b1=max(1,i1-wind1)
        e1=min(n1,i1+wind1)
        sz1=e1-b1+1
        temp_v=reshape(input(b1:e1,b2:e2,b3:e3),(/sz1*sz2*sz3/))
        call quick_sort(temp_v,sz1*sz2*sz3)
!                                write(0,*) "points",i1,i2,i3,temp_v(1:sz1*sz2*sz3)
        center=ceiling(.5*sz1*sz2*sz3)        
        output(i1,i2)=temp_v(center)
      end do
    end do 
    write(0,*) "before srite"
    if (n1*n2*4.ne.srite("out",output,n1*n2*4)) then
      call erexit("trouble writing outing array")
    end if
  end do 
end program  
