!$=head1 NAME
!$
!$Gauss - Make gaussian anomalies
!$
!$=head1 SYNOPSIS
!$
!$Gauss.x   >out.H pars
!$
!$=head1 INPUT PARAMETERS
!$
!$=over 4
!$
!$=item velback - real
!$
!$     [0.] background velocity
!$
!$=item n1 - integer
!$
!$     [1]  first axis dimension
!$
!$=item n2 - integer
!$
!$      [1]  second axis dimension
!$
!$=item n3- integer
!$
!$     [1]  third axis dimension
!$
!$=item o1- real
!$
!$      [0.]  first axis origin
!$
!$=item o2- real
!$
!$      [0.]  second axis origin
!$
!$=item o3- real
!$
!$      [0.]  third axis origin
!$
!$=item d1      - real
!$
!$      [1.]  first axis sampling
!$
!$=item d2      - real
!$
!$      [1.]  second axis sampling
!$
!$=item d3      - real
!$
!$      [1.]  third axis sampling
!$
!$=item ngauss  - integer
!$
!$      [1]  number of gaussian anomalies
!$
!$=item maxvel  - real
!$
!$      [1.]  maximum deviation from background
!$
!$=item var     - real
!$
!$      [1.]  variance  of gaussian anomalies
!$
!$=item max1    - real
!$
!$      [center of axis 1]location of maximum along 1 dimension
!$
!$=item max2    - real
!$
!$      [center of axis 2]location of maximum along 2 dimension
!$
!$=item max3    - real
!$
!$      [center of axis 3]location of maximum along 3 dimension
!$
!$=item zero_at -real
!$
!$      distance to zero anomaly
!$
!$=back
!$
!$=head1 OUTPUT PARAMETERS
!$
!$=over 4
!$
!$=item out.H - sepfile
!$
!$      SEPlib file with gaussian model
!$
!$=back
!$
!$
!$=head1 DESCRIPTION
!$
!$Makes simple Gaussian anomailes
!$
!$=head1 CATEGORY
!$
!$B<seis/model>
!$
!$=cut
!>
!-------------------------------------------------
!
!Author: Robert Clapp, ESMB 463, 7230253
!
!Date Created:Wed Oct  1 22:55:02 1997
!
!Purpose:
!
program Gauss 
  implicit none
  integer n1,n2,n3
  real o1,o2,o3,d1,d2,d3,velback,dist1,dist2,dist3,distance,zero_at
  integer i1,i2,i3,ngauss,ierr,igauss
  real,dimension(:,:), allocatable :: array
  real,allocatable :: max1(:),max2(:),max3(:),maxvel(:),var(:)
!READ IN THE PARAMETERS
  integer getch,putch
  call initpar()
call sep_add_doc_line("NAME")
call sep_add_doc_line("    Gauss - Make gaussian anomalies")
call sep_add_doc_line("")
call sep_add_doc_line("SYNOPSIS")
call sep_add_doc_line("    Gauss.x >out.H pars")
call sep_add_doc_line("")
call sep_add_doc_line("INPUT PARAMETERS")
call sep_add_doc_line("    velback - real")
call sep_add_doc_line("             [0.] background velocity")
call sep_add_doc_line("")
call sep_add_doc_line("    n1 - integer")
call sep_add_doc_line("             [1]  first axis dimension")
call sep_add_doc_line("")
call sep_add_doc_line("    n2 - integer")
call sep_add_doc_line("              [1]  second axis dimension")
call sep_add_doc_line("")
call sep_add_doc_line("    n3- integer")
call sep_add_doc_line("             [1]  third axis dimension")
call sep_add_doc_line("")
call sep_add_doc_line("    o1- real")
call sep_add_doc_line("              [0.]  first axis origin")
call sep_add_doc_line("")
call sep_add_doc_line("    o2- real")
call sep_add_doc_line("              [0.]  second axis origin")
call sep_add_doc_line("")
call sep_add_doc_line("    o3- real")
call sep_add_doc_line("              [0.]  third axis origin")
call sep_add_doc_line("")
call sep_add_doc_line("    d1 - real")
call sep_add_doc_line("              [1.]  first axis sampling")
call sep_add_doc_line("")
call sep_add_doc_line("    d2 - real")
call sep_add_doc_line("              [1.]  second axis sampling")
call sep_add_doc_line("")
call sep_add_doc_line("    d3 - real")
call sep_add_doc_line("              [1.]  third axis sampling")
call sep_add_doc_line("")
call sep_add_doc_line("    ngauss - integer")
call sep_add_doc_line("              [1]  number of gaussian anomalies")
call sep_add_doc_line("")
call sep_add_doc_line("    maxvel - real")
call sep_add_doc_line("              [1.]  maximum deviation from background")
call sep_add_doc_line("")
call sep_add_doc_line("    var - real")
call sep_add_doc_line("              [1.]  variance  of gaussian anomalies")
call sep_add_doc_line("")
call sep_add_doc_line("    max1 - real")
call sep_add_doc_line("              [center of axis 1]location of maximum along 1 dimension")
call sep_add_doc_line("")
call sep_add_doc_line("    max2 - real")
call sep_add_doc_line("              [center of axis 2]location of maximum along 2 dimension")
call sep_add_doc_line("")
call sep_add_doc_line("    max3 - real")
call sep_add_doc_line("              [center of axis 3]location of maximum along 3 dimension")
call sep_add_doc_line("")
call sep_add_doc_line("    zero_at -real")
call sep_add_doc_line("              distance to zero anomaly")
call sep_add_doc_line("")
call sep_add_doc_line("OUTPUT PARAMETERS")
call sep_add_doc_line("    out.H - sepfile")
call sep_add_doc_line("              SEPlib file with gaussian model")
call sep_add_doc_line("")
call sep_add_doc_line("DESCRIPTION")
call sep_add_doc_line("    Makes simple Gaussian anomailes")
call sep_add_doc_line("")
call sep_add_doc_line("CATEGORY")
call sep_add_doc_line("    seis/model")
call sep_add_doc_line("")

  call doc('SOURCE')
    if (0>= getch('n1','d',n1 )) then
      n1 = 1
    end if
    if (0.ne.putch('Read  from param: #n1 ','d',n1)) then
      call erexit('Trouble writing n1 to history')
    end if
    if (0>= getch('n2','d',n2 )) then
      n2 = 1
    end if
    if (0.ne.putch('Read  from param: #n2 ','d',n2)) then
      call erexit('Trouble writing n2 to history')
    end if
    if (0>= getch('n3','d',n3 )) then
      n3 = 1
    end if
    if (0.ne.putch('Read  from param: #n3 ','d',n3)) then
      call erexit('Trouble writing n3 to history')
    end if
    if (0>= getch('ngauss','d',ngauss )) then
      ngauss = 1
    end if
    if (0.ne.putch('Read  from param: #ngauss ','d',ngauss)) then
      call erexit('Trouble writing ngauss to history')
    end if
    if (0>= getch('velback','f',velback )) then
      velback = 0.
    end if
    if (0.ne.putch('Read  from param: #velback ','f',velback)) then
      call erexit('Trouble writing velback to history')
    end if
    if (0>= getch('o1','f',o1 )) then
      o1 = 0.
    end if
    if (0.ne.putch('Read  from param: #o1 ','f',o1)) then
      call erexit('Trouble writing o1 to history')
    end if
    if (0>= getch('o2','f',o2 )) then
      o2 = 0.
    end if
    if (0.ne.putch('Read  from param: #o2 ','f',o2)) then
      call erexit('Trouble writing o2 to history')
    end if
    if (0>= getch('o3','f',o3 )) then
      o3 = 0.
    end if
    if (0.ne.putch('Read  from param: #o3 ','f',o3)) then
      call erexit('Trouble writing o3 to history')
    end if
    if (0>= getch('d1','f',d1 )) then
      d1 = 1.
    end if
    if (0.ne.putch('Read  from param: #d1 ','f',d1)) then
      call erexit('Trouble writing d1 to history')
    end if
    if (0>= getch('d2','f',d2 )) then
      d2 = 1.
    end if
    if (0.ne.putch('Read  from param: #d2 ','f',d2)) then
      call erexit('Trouble writing d2 to history')
    end if
    if (0>= getch('d3','f',d3 )) then
      d3 = 1.
    end if
    if (0.ne.putch('Read  from param: #d3 ','f',d3)) then
      call erexit('Trouble writing d3 to history')
    end if
  allocate(maxvel(ngauss),max1(ngauss),max2(ngauss),max3(ngauss),var&
    &(ngauss))
  ierr=getch("maxvel","f",maxvel)
  if (ierr.eq.0) then
    maxvel=1.
  else if (ierr.ne.ngauss) then
    call erexit("number of maxvel not equal to ngauss")
  end if
  ierr=getch("max1","f",max1)
  if (ierr.eq.0) then
    max1=.5*(2*o1+d1*(n1-1))
  else if (ierr.ne.ngauss) then
    call erexit("number of max1 not equal to ngauss")
  end if
  ierr=getch("max2","f",max2)
  if (ierr.eq.0) then
    max2=.5*(2*o2+d2*(n2-1))
  else if (ierr.ne.ngauss) then
    call erexit("number of max2 not equal to ngauss")
  end if
  ierr=getch("max3","f",max3)
  if (ierr.eq.0) then
    max3=.5*(2*o3+d3*(n3-1))
  else if (ierr.ne.ngauss) then
    call erexit("number of max3 not equal to ngauss")
  end if
  ierr=getch("var","f",var)
  if (ierr.eq.0) then
    var=1.
  else if (ierr.ne.ngauss) then
    call erexit("number of var not equal to ngauss")
  end if
    if (0>= getch('zero_at','f',zero_at )) then
      zero_at = 100000000000000000.
    end if
    if (0.ne.putch('Read  from param: #zero_at ','f',zero_at)) then
      call erexit('Trouble writing zero_at to history')
    end if
!CHECK TO MAKE SURE ANOMALY CENTERS INSIDE MODEL
  if (minval(max1) < o1 .and. n1>1) then
    write(0,*) minval(max1) 
    call erexit("max1 value less than o1")
  end if
  if (maxval(max1) > o1 + d1 * (n1-1) .and. n1>1) then
    write(0,*) maxval(max1) 
    call erexit("max1 value more than o1 + d1 * (n1-1)")
  end if
  if (minval(max2) < o2 .and. n2>1) then
    write(0,*) minval(max2) 
    call erexit("max2 value less than o2")
  end if
  if (maxval(max2) > o2 + d2 * (n2-1) .and. n2.eq.1) then
    write(0,*) maxval(max2) 
    call erexit("max2 value more than o2 + d2 * (n2-1)")
  end if
  if (minval(max3) < o3 .and. n3>1) then
    write(0,*) minval(max3) ,o3
    call erexit("max1 value less than o3")
  end if
  if (maxval(max1) > o3 + d3 * (n3-1) .and. n3>1) then
    write(0,*) maxval(max3) ,o3+d3*(n3-1)
    call erexit("max1 value more than o3 + d3 * (n3-1)")
  end if
  allocate(array(n1,n2),stat=ierr)
  if (ierr.ne.0) then
    call erexit("trouble allocating 2-D array")
  end if
!WRITE OUT PARAMETERS
!        to history: integer n1:n1,n2:n2,n3:n3,esize:4
!        to history: real o1:o1,d1:d1,d2:d2,o2:o2,o3:o3,d3:d3
  call sep_put_data_axis_par("out",1,n1,o1,d1," ")
  call sep_put_data_axis_par("out",2,n2,o2,d2," ")
  call sep_put_data_axis_par("out",3,n3,o3,d3," ")
  do i3=1,n3 
    array=velback
    do igauss=1,ngauss 
      dist3=max3(igauss)-(o3+d3*(i3-1))
      do i2=1,n2 
        dist2=max2(igauss)-(o2+d2*(i2-1))
        do i1=1,n1 
          dist1=max1(igauss)-(o1+d1*(i1-1))
          distance=sqrt(dist1**2 + dist2**2 + dist3**2)
!                                        write(0,*) "distance",distance
          if (distance< zero_at) then
            array(i1,i2)=                                                array(i1,i2)+ maxvel(igauss) * exp(-.5*&
              & distance/sqrt(var(igauss)))
          end if
        end do
      end do
    end do 
    call srite("out",array,n1*n2*4)
  end do 
end program  
