!$=head1 NAME
!$
!$    Rtm2d-- Reverse time migration or two-way wave equation forward modeling
!$                                  
!$=head1 SYNOPSIS
!$                                  
!$    Rtm2d par=parfile.P wavelet=wavelet.H Slow=slowness.H Data=data.H
!$       Image_X=image_x.H Image_Z=image_z.H
!$                                                               
!$                                            
!$=head1 PARAMETERS
!$                                            
!$=over 4
!$=item migrationormodel - char*
!$
!$   [modeling]  :to do forward modeling
!$                                             
!$  migration:to do reverse time migration
!$                                             
!$=item extrapolate_oper- char *
!$                                             
!$     [fd]       : finite-difference method
!$                                              
!$    fft       : psedu-spectral method
!$                                              
!$    write_wave=y              : whether to write wavefield to harddisk or not$
!$                                               
!$=item    image_X  - sepfile 
!$                                               
!$    [optional] horizontal Common Image Gathers
!$                                               
!$=item image_Z  - sepfile 
!$                                               
!$    [optional]  vertical Common Image Gathers
!$                                               
!$=item      o_source  d_source n_source- real, real, integer
!$
!$  [0.,10.,1]   sampling of Sources
!$
!$=item      dt, nt_source - real, integer
!$
!$  [1500,.002]   sampling in time
!$
!$
!$=item      hx_min, dx_offset, n_off - real, real, integer
!$
!$  [-100.,10.,200]   sampling of offset
!$
!$=item dz_image, nz_image : Sampling parameters for depth
!$
!$=item z_source, z_receiver: depth of source and receiver
!$                                            
!$=item image_nhx, image_nhz: horizontal and vertical subsurface offset number of
!$                                           
!$=item images x_min_wave, x_max_wave: aperture of the wavefield
!$                                          
!$=item xcoe_taper, zcoe_taper: tapering boundary condition parameters
!$
!$=item nx_bd_taper, z_bd_taper: thickness of tapering boundary condtion
!$
!$=item nop  : thickness of one-way wave equation boundary condtion
!$
!$=item x_min_image, x_max_image: aperture of the image
!$
!$=item verb - logical
!$
!$ [.false.] Whether or not to print progress messages
!$
!$=item pct_master - real
!$
!$ [2.]  How often to print when master process
!$
!$=item pct_slave - real
!$
!$ [10.] How often to print when non-master process
!$
!$
!$=head1 CATEGORY
!$
!$B<seis/model>
!$
!$=cut
program rtm2d
use extrapolation_mpi_mod
use sep_par_mpi
use sep

implicit none
integer :: i_node,n_node,ierr
integer, external :: sep_thread_num
logical stat

call sep_add_doc_line("NAME")
call sep_add_doc_line("        Rtm2d-- Reverse time migration or two-way wave equation forward modeling")
call sep_add_doc_line("")
call sep_add_doc_line("SYNOPSIS")
call sep_add_doc_line("        Rtm2d par=parfile.P wavelet=wavelet.H Slow=slowness.H Data=data.H")
call sep_add_doc_line("           Image_X=image_x.H Image_Z=image_z.H")
call sep_add_doc_line("")
call sep_add_doc_line("PARAMETERS")
call sep_add_doc_line("           [modeling]  :to do forward modeling")
call sep_add_doc_line("")
call sep_add_doc_line("          migration:to do reverse time migration")
call sep_add_doc_line("")
call sep_add_doc_line("    extrapolate_oper- char *")
call sep_add_doc_line("             [fd]       : finite-difference method")
call sep_add_doc_line("")
call sep_add_doc_line("            fft       : psedu-spectral method")
call sep_add_doc_line("")
call sep_add_doc_line("            write_wave=y              : whether to write wavefield to harddisk or not$")
call sep_add_doc_line("")
call sep_add_doc_line("    image_X - sepfile")
call sep_add_doc_line("            [optional] horizontal Common Image Gathers")
call sep_add_doc_line("")
call sep_add_doc_line("    image_Z - sepfile")
call sep_add_doc_line("            [optional]  vertical Common Image Gathers")
call sep_add_doc_line("")
call sep_add_doc_line("    o_source d_source n_source- real, real, integer")
call sep_add_doc_line("          [0.,10.,1]   sampling of Sources")
call sep_add_doc_line("")
call sep_add_doc_line("    dt, nt_source - real, integer")
call sep_add_doc_line("          [1500,.002]   sampling in time")
call sep_add_doc_line("")
call sep_add_doc_line("    hx_min, dx_offset, n_off - real, real, integer")
call sep_add_doc_line("          [-100.,10.,200]   sampling of offset")
call sep_add_doc_line("")
call sep_add_doc_line("    dz_image, nz_image : Sampling parameters for depth")
call sep_add_doc_line("    z_source, z_receiver: depth of source and receiver")
call sep_add_doc_line("    image_nhx, image_nhz: horizontal and vertical subsurface offset number")
call sep_add_doc_line("    of")
call sep_add_doc_line("    images x_min_wave, x_max_wave: aperture of the wavefield")
call sep_add_doc_line("    xcoe_taper, zcoe_taper: tapering boundary condition parameters")
call sep_add_doc_line("    nx_bd_taper, z_bd_taper: thickness of tapering boundary condtion")
call sep_add_doc_line("    nop : thickness of one-way wave equation boundary condtion")
call sep_add_doc_line("    x_min_image, x_max_image: aperture of the image")
call sep_add_doc_line("    verb - logical")
call sep_add_doc_line("         [.false.] Whether or not to print progress messages")
call sep_add_doc_line("")
call sep_add_doc_line("    pct_master - real")
call sep_add_doc_line("         [2.]  How often to print when master process")
call sep_add_doc_line("")
call sep_add_doc_line("    pct_slave - real")
call sep_add_doc_line("         [10.] How often to print when non-master process")
call sep_add_doc_line("")
call sep_add_doc_line("CATEGORY")
call sep_add_doc_line("        seis/model")
call sep_add_doc_line("")

call sep_par_start()

stat=extrapolate()

call extrapolation_clean()
call sep_par_stop()

end program rtm2d
