/*$

=head1 NAME

Transp - transpose a SEPlib dataset in plane of choice

=head1 SYNOPSIS

Transp  <in.H >out.H 

=head1 INPUT PARAMETERS

=over 4

=item plane - int 

       [12]  Plane to transpose

=item max_memory/maxsize  -  int 

       [10]  Maximum memory to use in  megabytes

=item verb - int

       [0]  Whether (1) or not (0) to be verbose

=item  reshape  - int*

       [none] Treat the cube dimensions differently

=back

=head1 DESCRIPTION 
 
Transp- transposes a SEPlib dataset in the plane specified.

=head1 COMMENTS

  The reshape option allows for transposes that can't be
  strictly described by a plane. For example if you
  want to change 1,2,3,4,5 to 5,3,4,1,2 this is really
  a transpose of a cube where axes 1 and 2 are combined
  and axes 3 and 4 are combined. To do this transpose
  operation you would specify plane=13 and reshape=2,4,5


=head1 CATEGORY

B<util/cube>

=cut

DISTR
>*/ 
/*
-------------------------------------------------

Author: Robert Clapp, ESMB 463, 7230253

Date Created:Fri Feb 18 10:11:05 PST 2000

Purpose: 

*/	 

#define MY_SEP_DOC \
 sep_add_doc_line("NAME");\
 sep_add_doc_line("    Transp - transpose a SEPlib dataset in plane of choice");\
 sep_add_doc_line("");\
 sep_add_doc_line("SYNOPSIS");\
 sep_add_doc_line("    Transp <in.H >out.H");\
 sep_add_doc_line("");\
 sep_add_doc_line("INPUT PARAMETERS");\
 sep_add_doc_line("    plane - int");\
 sep_add_doc_line("               [12]  Plane to transpose");\
 sep_add_doc_line("");\
 sep_add_doc_line("    max_memory/maxsize - int");\
 sep_add_doc_line("               [10]  Maximum memory to use in  megabytes");\
 sep_add_doc_line("");\
 sep_add_doc_line("    verb - int");\
 sep_add_doc_line("               [0]  Whether (1) or not (0) to be verbose");\
 sep_add_doc_line("");\
 sep_add_doc_line("    reshape - int*");\
 sep_add_doc_line("               [none] Treat the cube dimensions differently");\
 sep_add_doc_line("");\
 sep_add_doc_line("DESCRIPTION");\
 sep_add_doc_line("    Transp- transposes a SEPlib dataset in the plane specified.");\
 sep_add_doc_line("");\
 sep_add_doc_line("COMMENTS");\
 sep_add_doc_line("      The reshape option allows for transposes that can't be");\
 sep_add_doc_line("      strictly described by a plane. For example if you");\
 sep_add_doc_line("      want to change 1,2,3,4,5 to 5,3,4,1,2 this is really");\
 sep_add_doc_line("      a transpose of a cube where axes 1 and 2 are combined");\
 sep_add_doc_line("      and axes 3 and 4 are combined. To do this transpose");\
 sep_add_doc_line("      operation you would specify plane=13 and reshape=2,4,5");\
 sep_add_doc_line("");\
 sep_add_doc_line("CATEGORY");\
 sep_add_doc_line("    util/cube");\
 sep_add_doc_line("");
#include <sep.main> 
#include<superset.h>
#define INPUT 0
#define OUTPUT 0



      int read_in[5],rite_in[5],read_out[5],rite_out[5],looping;
char temp_file[1024];
int esize;
static int ith=0;

#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int read_pars(int *verb, int *plane,int *mem,int *ndim, int *pipe_in, int *pipe_out,int *mmap_it, int *nreshape_dims, int *reshape_dims);
int check_pars(int verb,int *plane,int mem, int ndim,int nreshape_dims, int *reshape_dims);
int rite_pars(int *plane, int mem,int nreshape_dims,int *reshape_dims);
int standardize_it(int verb, int *plane, int mem, int *nin, int *nout,int ndim,int pipein,int  *pipeout, int mmap_it,int nreshape_dim,int *reshape_dims);

int transp_it(int verb,int *nin, int *nout,int mem,int pipein, int pipeout);
int transp_reg(char *tagin,char *tagout,int f5in,int f5out,int nread,int *rite_in,int *rite_out,int *nin,int *nout,char *block_in,char *block_out,int verb);
_XFUNCPROTOEND
#else
int read_pars();
int check_pars();
int rite_pars();
int standardize_it();
int transp_it();
int transp_reg();
#endif


MAIN()
{ 

  int n1;
	int verb,mem,ndim;
	int plane[2],pipe_in,pipe_out;
	int nin[5],nout[5],mmap_it;
  int nreshape_dim,reshape_dims[30];
  char *array;
   init_3d();

	if(SUCCESS!=read_pars(&verb,plane,&mem,&ndim,&pipe_in,&pipe_out,&mmap_it,
    &nreshape_dim,reshape_dims))
		seperr("trouble reading parameters \n");
	if(SUCCESS!=check_pars(verb,plane,mem,ndim,nreshape_dim,reshape_dims))
		seperr("trouble checking paramters\n");
	if(SUCCESS!=rite_pars(plane,mem,nreshape_dim,reshape_dims))
		seperr("trouble writing parameters \n");
	if(SUCCESS!=standardize_it(verb,plane,mem,nin,nout,
    ndim,pipe_in,&pipe_out,mmap_it,nreshape_dim,reshape_dims))
		seperr("trouble writing parameters \n");

	if(SUCCESS!=transp_it(verb,nin,nout,mem, pipe_in,pipe_out)){
		seperr("trouble transposing \n");
	}
	
	return(0);
 
 } 
#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int read_pars(int *verb,int *plane, int *mem, int *ndim,int *pipein, int *pipeout, int *mmap_it,int *nreshape_dim, int *reshape_dims)
_XFUNCPROTOEND
#else
int read_pars(verb,plane,mem,ndim,pipein,pipeout,mmap_it,nreshape_dim,reshape_dims)
int *verb,*plane,*mem,*ndim,*pipein,*pipeout,*mmap_it,*nreshape_dim,*reshape_dims;
#endif
{
char pl[2],temp_ch[3];

if(0==getch("verb","d",verb)) *verb=0;

*pipein=sseekable("in");
*pipeout=sseekable("out");
if(*verb==1){ 
	fprintf(stderr,"input seekable=%d output seekable=%d \n",*pipein,*pipeout);
}


if(0==getch("mmap_it","d",mmap_it)) *mmap_it=0;
if(0==getch("max_memory","d",mem)){
  if(0==getch("maxsize","d",mem)) *mem=10;
}


if(SUCCESS!=sep3d_tag_init("in","in","INPUT"))    
	seperr("trouble initializing tag %s \n","in");

if(SUCCESS!=sep3d_grab_ndims("in",ndim))    
	seperr("trouble  reading ndim from tag in \n");

if(0==getch("plane","s",pl)) strcpy(pl,"12");
	sprintf(temp_ch,"%c",pl[0]); plane[0]=atoi(temp_ch);
	sprintf(temp_ch,"%c",pl[1]); plane[1]=atoi(temp_ch);

*nreshape_dim=getch("reshape","d",reshape_dims);
 
return(0);


}

#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int check_pars(int verb,int *plane, int mem, int ndim,int nre,int *redims)
_XFUNCPROTOEND
#else
int check_pars(verb,plane,mem,ndim,nre,redims)
int verb,mem,ndim,*plane,nre,*redims;
#endif
{
char temp_ch[1024];
int tempi,nt[99];
float o,d;
char label[128],unit[128];
int n[2],i;
int ndim_use;

/* Start with the stupidity checks*/
if(mem< 1) seperr("memory must be 1 or greater \n");



/*make first plane smaller axis for convenience*/
if(plane[0] >plane[1]){ tempi=plane[1]; plane[1]=plane[0];plane[0]=tempi;}

if(plane[0] < 1)
	seperr("Bad plane argument, plane=%d data limits 0 to %d \n",plane[0],ndim);
if(plane[1] < 1)
	seperr("Bad plane argument, plane=%d data limits 0 to %d \n",plane[1],ndim);
if(plane[0] == plane[1])
	seperr("Can't transpose a plane with itself \n");

if(SUCCESS!=sep3d_grab_file_type("in",temp_ch))
    seperr("trouble figuring out file type for tag %s \n","in");

/*RESHAPE WILL BE ARE RESHAPED DIMENSIONS*/
if(SUCCESS!=sep3d_struct_init("in","reshape","SCRATCH"))
   seperr("trouble creating reshape structure \n");

if(nre>0){
    if(SUCCESS!=sep3d_change_dims("reshape",nre,redims))
		  seperr("trouble resetting number of dimensions in reshape structure\n");
  ndim_use=nre;
}
ndim_use=ndim;

if(0!=strcmp(temp_ch,"REGULAR"))
	seperr("Use Sort3d plane=%c%c for a non-SEP regular dataset\n",plane[0],plane[1]);

if(0==hetch("esize","d",&esize)) esize=4;
tempi=esize;


if(SUCCESS!=sep3d_grab_axis("reshape",plane[0],&n[0],&o,&d,label,unit))
	seperr("trouble reading axis %d from in \n",plane[0]);

if(nre>0 && plane[1] > ndim)
  seperr("can't reshape and increase dimensionality\n");

if(plane[1]>ndim_use){
	for(i=0; i < ndim;i++) nt[i]=i+1; 
	for(i=ndim; i < plane[1];i++) nt[i]=ndim;

	if(SUCCESS!=sep3d_change_dims("in",plane[1],nt))
		seperr("trouble resetting number of diemensions \n");
	if(SUCCESS!=sep3d_change_dims("reshape",plane[1],nt))
		seperr("trouble resetting number of diemensions \n");
}
for(i=ndim; i < plane[1]; i++){

  sprintf(temp_ch,"o%d",i+1); o=0.;hetch(temp_ch,"f",&o);
  sprintf(temp_ch,"d%d",i+1); d=1.;hetch(temp_ch,"f",&d);
  sprintf(temp_ch,"label%d",i+1); strcpy(label,"none");hetch(temp_ch,"s",label);
  sprintf(temp_ch,"unit%d",i+1); strcpy(unit,"none");hetch(temp_ch,"s",unit);
  if(SUCCESS!=sep3d_set_axis("in",i+1,1,o,d,label,unit))
	   seperr("trouble reading axis %d from in \n",i);
}


if(SUCCESS!=sep3d_grab_axis("reshape",plane[1],&n[1],&o,&d,label,unit))
	seperr("trouble reading axis %d from in \n",plane[1]);


if(verb>0){
	fprintf(stderr,"Transposing plane %d and %d \n",plane[0],plane[1]);
	fprintf(stderr,"Using %d megabytes\n",mem);
}

return(0);

}

#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int rite_pars(int *plane, int mem,int nre, int *redims)
_XFUNCPROTOEND
#else
int rite_pars(plane,mem,nre,redims)
int *plane,mem,nre,*redims;
#endif
{
int i1,ndim,i,ir;
char label[128],unit[128];
int  n,ierr,nr;
float o,d;
char temp_ch[128];
int iplane_in[20];
int ifirst_out[20];
int icount[20],ifirst_in[20];

puthead("Transposing plane %d and %d \n",plane[0],plane[1]);
puthead("Using %d megabytes \n",mem);

/*create output dataset */
if(SUCCESS!=sep3d_struct_init("in","out","OUTPUT"))
   seperr("trouble creating output structure \n");



/*FIGURE OUT WHAT INPUT AXIS BELONGS TO THE RESHAPED AXIS*/
if(SUCCESS!=sep3d_grab_ndims("reshape",&nr))    
	seperr("trouble  reading ndim from tag in \n");
if(SUCCESS!=sep3d_grab_ndims("in",&ndim))    
	seperr("trouble  reading ndim from tag in \n");

for(i=0; i< 20; i++) icount[i]=0;

/*loop over the input figure out what reshaped space it belongs to*/
for(i=0; i < ndim; i++){
  if(nre<1) iplane_in[i]=i+1;
  else{
    iplane_in[i]=1;
    ir=0;while(redims[ir] < i+1) ir++;
    iplane_in[i]=ir+1;
  }
  if(icount[iplane_in[i]]==0) ifirst_in[iplane_in[i]]=i+1;
  icount[iplane_in[i]]++;
}

/*record the begining output axis for each reshaped portion*/
for(i=1; i <= plane[0]; i++) ifirst_out[i]=ifirst_in[i];
for(i=plane[1]; i <= nr; i++) ifirst_out[i]=ifirst_in[i];
ifirst_out[plane[1]]=ifirst_in[plane[0]];
if(plane[1]-plane[0] >1 ){
  ifirst_out[plane[0]+1]=ifirst_in[plane[0]]+icount[plane[1]];
  for(i=plane[0]+2; i <plane[1] ; i++) ifirst_out[i]=icount[i-1]+ifirst_out[i-1];
  ifirst_out[plane[0]]=ifirst_out[plane[1]-1]+icount[plane[1]-1];
}
else ifirst_out[plane[0]]=ifirst_out[plane[1]]+icount[plane[1]];


for(i=0; i < ndim; i++){
   i1=i+1;
   if(SUCCESS!=sep3d_grab_axis("in",i1,&n,&o,&d,label,unit))
	   seperr("trouble reading axis %d from in \n",i1);
   
   i1=ifirst_out[iplane_in[i]];
   if(SUCCESS!=sep3d_set_axis("out",i1,n,o,d,label,unit))
	   seperr("trouble reading axis %d from in \n",i1);
   ifirst_out[iplane_in[i]]++;  
}

if(SUCCESS!=sep3d_rite_format("out","out"))
	seperr("trouble writing out dataset parameters \n");
return(0);
}
#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int standardize_it(int verb, int *plane, int mem,  int *nin, int *nout,int ndim,int pipein, int *pipeout,int mmap_it,int nre,int *redims)
_XFUNCPROTOEND
#else
int standardize_it(verb,plane,mem,nin,nout,ndim,pipein,pipeout,mmap_it,nre,redims)
int *plane,mem,*nin,*nout,ndim,pipein,*pipeout,mmap_it,nre,*redims;
#endif
{
int n,nr;
float o,d;
double memd;
int axis[6],tempi,i,itemp;
double used,use2,tot[5];
char label[128], unit[128],temp_ch[128];




/*                                                   */
/*  Let us reshape the input and output files        */
/*                                                   */
axis[0]=plane[0]-1;
axis[1]=plane[0];
axis[2]=plane[1]-1;
axis[3]=plane[1];
if(nre!=0) axis[4]=MAX(nre,plane[1]);
else axis[4]=MAX(ndim,plane[1]);




if(SUCCESS!=sep3d_grab_ndims("reshape",&nr))    
	seperr("trouble  reading ndim from tag in \n");
for(i=0;i<nr;i++){
  itemp=i+1;
	if(SUCCESS!=sep3d_grab_axis("reshape",itemp,&nin[i],&o,&d,label,unit))
		seperr("trouble grabbing axis %d from tag : in \n",i+1);
}
  

if(SUCCESS!=sep3d_change_dims("reshape",5,axis))
	seperr("trouble resetting dims for tag: in \n");

for(i=0; i < 5; i++){
	if(SUCCESS!=sep3d_grab_axis("reshape",i+1,&nin[i],&o,&d,label,unit))
		seperr("trouble grabbing axis %d from tag : in \n",i+1);
  nout[i]=nin[i];
}
nout[3]=nin[1];
nout[1]=nin[3];



/*                                                   */
/*   Now lets see if we have to create a temp file   */
/*                                                   */

used=esize;
memd=(double)mem*1000000.;

tot[0]=(double)nin[0]*used; tot[1]=(double)nin[1]*tot[0]; tot[2]=(double)nin[2]*tot[1];
tot[3]=(double)nin[3]*tot[2]; tot[4]=(double)nin[4]*tot[3];


strcpy(temp_file,"in");

if((tot[3] *2. >memd && pipein==1 && *pipeout==1 ) && mmap_it==0 ){
	if(verb>0) fprintf(stderr,"I am going to have create a temporary file \n");
	itemp=1;
	strcpy(temp_file,"TEMP_XXXXXX"); mktemp(temp_file);
	auxtmp(temp_file);
/*	if(NULL==auxtmp(temp_file))*/
/*		seperr("trouble creating temporary file \n");*/
}
else{
 if(verb>0) fprintf(stderr,"I don't have to create a temporary file \n");
	itemp=0;
	*pipeout=0;
}

/* The logic is going to be:
     Given a choice loop the output 

*/

if(pipein==1){ /* if the input is a pipe we need to loop the input */
	looping=INPUT;
	if(tot[0]>memd) { 
		read_in[0]=MAX(1,(int)(memd/used)); 
		read_in[1]=1; read_in[2]=1; read_in[3]=1; read_in[4]=1;
	}
	else if(tot[1] > memd){
		read_in[0]=nin[0]; read_in[1]=MAX(1,(int)(memd/tot[0])); 
		read_in[2]=1; read_in[3]=1; read_in[4]=1;
	}
	else if(tot[2] > memd){
		read_in[0]=nin[0]; read_in[1]=nin[1]; 
		read_in[2]=MAX(1,(int)(memd/tot[1]));
		read_in[3]=1; read_in[4]=1;
	}
	else if(tot[3]*2 > memd){
		read_in[0]=nin[0]; read_in[1]=nin[1]; 
		read_in[3]=MIN(MAX(1,(int)(memd/tot[2]/2.)),nin[3]);
		read_in[2]=nin[2]; read_in[4]=1;
	}
	else if(tot[4]*2 > memd){
		read_in[0]=nin[0]; read_in[1]=nin[1]; 
		read_in[4]=MAX(1,(int)(memd/tot[3]/2.));
		read_in[2]=nin[2]; read_in[3]=nin[3];
	}
	else{
		read_in[0]=nin[0]; read_in[1]=nin[1]; 
		read_in[4]=nin[4];
		read_in[2]=nin[2]; read_in[3]=nin[3];
	}

	if(*pipeout==0){
		read_out[0]=read_in[0]; read_out[1]=read_in[3]; read_out[2]=read_in[2];
		read_out[3]=read_in[1]; read_out[4]=read_in[4];

		rite_in[0]=read_in[0]; rite_in[1]=read_in[1]; rite_in[2]=read_in[2];
		rite_in[3]=read_in[3]; rite_in[4]=read_in[4];

		rite_out[0]=read_in[0]; rite_out[1]=read_in[3]; rite_out[2]=read_in[2];
		rite_out[3]=read_in[1]; rite_out[4]=read_in[4];
	}
	else{
		read_out[0]=read_in[0]; read_out[1]=read_in[1]; read_out[2]=read_in[2];
		read_out[3]=read_in[3]; read_out[4]=read_in[4];
	}
}

tot[0]=(double)nout[0]*used; tot[1]=(double)nout[1]*tot[0]; tot[2]=(double)nout[2]*tot[1];
tot[3]=(double)nout[3]*tot[2]; tot[4]=(double)nout[4]*tot[3];


if(!(pipein==1 && *pipeout==0)){ /*if we are looping the output */
	looping=OUTPUT;
	if(tot[0]>memd) { 
		rite_out[0]=MAX(1,(int)(memd/used)); 
		rite_out[1]=1; rite_out[2]=1; rite_out[3]=1; rite_out[4]=1;
	}
	else if(tot[1] > memd){
		rite_out[0]=nout[0]; rite_out[1]=MAX(1,(int)(memd/tot[0])); 
		rite_out[2]=1; rite_out[3]=1; rite_out[4]=1;
	}
	else if(tot[2] > memd){
		rite_out[0]=nout[0]; rite_out[1]=nout[1]; 
		rite_out[2]=MAX(1,(int)(memd/tot[1]));
		rite_out[3]=1; rite_out[4]=1;
	}
	else if(tot[3]*2 > memd){
		rite_out[0]=nout[0]; rite_out[1]=nout[1]; 
		rite_out[3]=MIN(MAX(1,(int)(memd/tot[2]/2.)),nout[3]);
		rite_out[2]=nout[2]; rite_out[4]=1;
	}
	else if(tot[4]*2 > memd){
		rite_out[0]=nout[0]; rite_out[1]=nout[1]; 
		rite_out[2]=nout[2]; rite_out[3]=nout[3];
		rite_out[4]=1;
	}
	else{
		rite_out[0]=nout[0]; rite_out[1]=nout[1]; 
		rite_out[2]=nout[2]; rite_out[3]=nout[3];
		rite_out[4]=nout[4];
	}

	rite_in[0]=rite_out[0]; rite_in[1]=rite_out[1]; rite_in[2]=rite_out[2];
	rite_in[3]=rite_out[3]; rite_in[4]=rite_out[4];
	
}
if(verb>0){
	fprintf(stderr,"I am doing my block read_in with %d %d %d %d %d \n",read_in[0],read_in[1],read_in[2],read_in[3],read_in[4]);
	fprintf(stderr,"I am doing my block read_out with %d %d %d %d %d \n",read_out[0],read_out[1],read_out[2],read_out[3],read_out[4]);
	fprintf(stderr,"I am doing my block rite_in with %d %d %d %d %d \n",rite_in[0],rite_in[1],rite_in[2],rite_in[3],rite_in[4]);
	fprintf(stderr,"I am doing my block rite_out with %d %d %d %d %d \n",rite_out[0],rite_out[1],rite_out[2],rite_out[3],rite_out[4]);
}

return(0);
}
#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int transp_it(int verb,int *nin, int *nout,int mem,int pipein, int pipeout)
_XFUNCPROTOBEGIN
#else    
int transp_it(verb,nin,nout, mem,pipein,pipeout)
int verb,*nin, *nout, mem,pipein,pipeout;
#endif
{
int done,nread,fread;
int i4,i3,i2,i1,n,ntot1,ntot2,ntot,frite;
char read_from[128];
char *block_in,*block_out;


	ntot1=rite_out[0]*rite_out[1]*rite_out[2]*rite_out[3]*rite_out[4]*esize;
	if(pipein==1)
		ntot2=read_in[0]*read_in[1]*read_in[2]*read_in[3]*read_in[4]*esize;
	else ntot2=0;

	ntot=MAX(ntot1,ntot2);



	block_in=(char*) alloc(ntot);
	block_out=(char*) alloc(ntot);


done=0;
while(done < nin[4]){
	nread=MIN(MAX(read_in[4],rite_out[4]),nin[4]-done);
 		fread=done;
	

	if(pipein==1 && pipeout==1){
		if(0!=strcmp(temp_file,"in")){
			if(SUCCESS!=transfer_block("in",temp_file,nin,fread,nread,ntot,block_in))
				seperr("trouble copying temp buffer \n");
		}
		if(SUCCESS!=transp_reg(temp_file,"out",0,fread,nread,rite_in,rite_out,
       nin,nout,block_in,block_out,verb))
			seperr("trouble transposing block \n");
	}
	else if(pipeout==0){
		if(SUCCESS!=transp_reg("in","out",fread,fread,nread,rite_in,rite_out,nin,
      nout,block_in,block_out,verb))
			seperr("trouble transposing regular block \n");
	}
	else{
		if(SUCCESS!=transp_reg("in","out",fread,fread,nread,rite_in,rite_out,nin,
      nout,block_in,block_out,verb))
			seperr("trouble transposing regular block \n");
	}
	done=done+nread;
	if(verb && nin[4]>1) fprintf(stderr,"Finished block %d of %d \n",done,nin[4]);
}
return(SUCCESS);
}
#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int transp_reg(char *tagin,char *tagout,int f5in,int f5out,int n5,int *rite_in,int *rite_out,int *nin,int *nout,char *block_in,char *block_out,int verb)
_XFUNCPROTOEND
#else    
int transp_reg(tagin,tagout,f5in,f5out,n5,rite_in,rite_out,nin,nout,block_in,block_out,verb)
char *tagin,*tagout;
int f5in, f5out,n5,*rite_in,*rite_out,*nin,*nout,verb;
char *block_in, *block_out;
#endif
{
int i1,i2,i3,i4,i5;
int dn[4];
int fread[5],frite[5],j[5],nread[5],nrite[5],ipos;
int n4,n3,n2,n1,in5,in4,in3,in2,in1,out1,out2,out3,out,out5,out4,five;
int ibeg,ihold,isend,iend;
float block[100];






fread[4]=f5in; frite[4]=f5out;five=5;
nread[4]=n5; nrite[4]=n5;
j[0]=1; j[1]=1; j[2]=1; j[3]=1; j[4]=1;
frite[0]=0; frite[1]=0; frite[2]=0; frite[3]=0;
while(frite[3] < nout[3]){
	nrite[3]=MIN(rite_out[3],nout[3]-frite[3]);	
	while(frite[2] < nout[2]){
		nrite[2]=MIN(rite_out[2],nout[2]-frite[2]);	
		while(frite[1] < nout[1]){
			nrite[1]=MIN(rite_out[1],nout[1]-frite[1]);	
			while(frite[0] < nout[0]){
				nrite[0]=MIN(rite_out[0],nout[0]-frite[0]);	
				nread[0]=nrite[0];nread[1]=nrite[3];nread[2]=nrite[2];nread[3]=nrite[1];
				fread[0]=frite[0];fread[1]=frite[3];fread[2]=frite[2];fread[3]=frite[1];




				ith++;
				if(0!=sreed_window(tagin,&five,nin,nread,fread,j,esize,block_in)){
					fprintf(stderr,"reading parameters ngrid-%d %d %d %d %d \n",nin[0],nin[1],nin[2],nin[3],nin[4]);
					fprintf(stderr,"reading parameters nwind-%d %d %d %d %d \n",nread[0],nread[1],nread[2],nread[3],nread[4]);
					fprintf(stderr,"reading parameters fwind-%d %d %d %d %d \n",fread[0],fread[1],fread[2],fread[3],fread[4]);
					fprintf(stderr,"reading parameters jwind-%d %d %d %d %d \n",j[0],j[1],j[2],j[3],j[4]);
					seperr("trouble reading block \n");
				}

					n4=nrite[3];n3=nrite[2];n2=nrite[1];n1=nrite[0];in1=esize;out1=esize;
					in2=n1*in1; out2=n1*out1; 
					in3=in2*n4; out3=out2*n2; 
					in4=in3*n3;out4=out3*n3;
					in5=in4*n2;out5=out4*n4;
					ipos=0;

					
					ibeg=0;
					/*can you believe all of this crap just do do this !!!!*/
					for(i5=ibeg; i5 < n5; i5++){
					for(i4=0; i4 < n4; i4++){
						for(i3=0; i3 < n3; i3++){
							for(i2=0; i2 < n2; i2++){
									memcpy( (void*)(block_out+i2*out2+i3*out3+i4*out4+out5*i5),
										(const void*)(block_in+i4*in2+i3*in3+i2*in4+in5*i5),
                 in2);
								  
							}
						}
					}}

					if(0!=srite_window(tagout,&five,nout,nrite,frite,j,esize,block_out))
						seperr("trouble reading block \n");
				frite[0]=frite[0]+nrite[0]; 
			}
			frite[1]=frite[1]+nrite[1];  frite[0]=0;
		}
		frite[2]=frite[2]+nrite[2]; frite[1]=0;
	}
	frite[3]=frite[3]+nrite[3]; frite[2]=0;
	if(nout[4]==1 && verb) fprintf(stderr,"Finished block %d of %d \n",frite[3],nout[3]);
}

return(SUCCESS);
}
#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int transfer_block(char *tagin,char *tagout, int *nin, int f5, int n5, int ntot, char *block_in)
_XFUNCPROTOEND
#else    
int transfer_block(tagin,tagout, nin, f5, n5,ntot,block_in)
char *tagin, *tagout,*block_in; 
int *nin, f5,  n5,ntot;
#endif    
{
double nbig;
double ndone;
int block;



if(f5!=0) sseek(tagout,0,0);
nbig=(double)nin[0]*(double)nin[1]*(double)nin[2]*(double)nin[3]*(double)esize;
ndone=0;

while(ndone< nbig){
	block=MIN((int)(nbig-ndone),ntot);
	if(block!=sreed(tagin,block_in,block))
		seperr("troulbe reading from tag=%s \n",tagin);
	if(block!=srite(tagout,block_in,block))
		seperr("troulbe writing to tag=%s \n",tagout);

	ndone=ndone+block;
}


return(SUCCESS);
}
#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int transp_it_mmap(int verb,int *nin, int *nout,int mem,int pipein, int pipeout)
_XFUNCPROTOBEGIN
#else    
int transp_it_mmap(verb,nin,nout, mem,pipein,pipeout)
int verb,*nin, *nout, mem,pipein,pipeout;
#endif
{
int done,nread,fread;
int i4,i3,i2,i1,n,ntot1,ntot2,ntot,frite;
char read_from[128];
char *block_in,*block_out;


	ntot1=rite_out[0]*rite_out[1]*rite_out[2]*rite_out[3]*rite_out[4]*esize;
	if(pipein==1)
		ntot2=read_in[0]*read_in[1]*read_in[2]*read_in[3]*read_in[4]*esize;
	else ntot2=0;

	ntot=MAX(ntot1,ntot2);

 /*create the temporary file*/
 


	block_in=(char*) alloc(ntot);
	block_out=(char*) alloc(ntot);
   


done=0;
while(done < nin[4]){
	nread=MIN(MAX(read_in[4],rite_out[4]),nin[4]-done);
 		fread=done;
	

	if(pipein==1 && pipeout==1){
		if(0!=strcmp(temp_file,"in")){
			if(SUCCESS!=transfer_block("in",temp_file,nin,fread,nread,ntot,block_in))
				seperr("trouble copying temp buffer \n");
		}
		if(SUCCESS!=transp_reg(temp_file,"out",0,fread,nread,rite_in,rite_out,
       nin,nout,block_in,block_out,verb))
			seperr("trouble transposing block \n");
	}
	else if(pipeout==0){
		if(SUCCESS!=transp_reg("in","out",fread,fread,nread,rite_in,rite_out,nin,
      nout,block_in,block_out,verb))
			seperr("trouble transposing regular block \n");
	}
	else{
		if(SUCCESS!=transp_reg("in","out",fread,fread,nread,rite_in,rite_out,nin,
      nout,block_in,block_out,verb))
			seperr("trouble transposing regular block \n");
	}
	done=done+nread;
	if(verb && nin[4]>1) fprintf(stderr,"Finished block %d of %d \n",done,nin[4]);
}

return(SUCCESS);
}

