/*
=head1 NAME

 Partition
  
=head1 SYNOPSIS

  Partition intag= outtag= axis=

=head1 PARAMTERS

=over 4

=item assign - integer*

 section number to assign from each thread (starting with thread 1) 
   
 When running in mpi mode len(assign)=nmpi-1

=item max_size - integer

 [100]  maximum memory in megabyte to use on node (max memory will then be 2x)


=item axis - integer

![2]  axis to distribute over

=item pattern - char*

  ['BLOCK']  Whether to distribute an axis in blocks or 'SPREAD' format

=item verb - logical

  [0]  Wether to be verbose


=head1 CATEGORY
                                                                                                        
<util/cube>
                                                                                                        


=cut

*/

#include<seplib.h>
#include<sep3dc.h>
#include<sepaux.h>
#include<superset.h>
#ifndef SEPNULL2
#define SEPNULL2 ((void *) 0)
#endif
#ifdef SEP_MPI
#include<mpi.h>
#endif

#define HEAD "/dev/null"
                                                                                                                     
#define MY_SEP_DOC \
 sep_add_doc_line("NAME");\
 sep_add_doc_line("     Partition");\
 sep_add_doc_line("");\
 sep_add_doc_line("SYNOPSIS");\
 sep_add_doc_line("      Partition intag= outtag= axis=");\
 sep_add_doc_line("");\
 sep_add_doc_line("PARAMTERS");\
 sep_add_doc_line("    assign - integer*");\
 sep_add_doc_line("         section number to assign from each thread (starting with thread 1) ");\
 sep_add_doc_line("");\
 sep_add_doc_line("         When running in mpi mode len(assign)=nmpi-1");\
 sep_add_doc_line("");\
 sep_add_doc_line("    max_size - integer");\
 sep_add_doc_line("         [100]  maximum memory in megabyte to use on node (max memory will then be 2x)");\
 sep_add_doc_line("");\
 sep_add_doc_line("    axis - integer");\
 sep_add_doc_line("        ![2] axis to distribute over");\
 sep_add_doc_line("");\
 sep_add_doc_line("    pattern - char*");\
 sep_add_doc_line("          ['BLOCK']  Whether to distribute an axis in blocks or 'SPREAD' format");\
 sep_add_doc_line("");\
 sep_add_doc_line("    verb - logical");\
 sep_add_doc_line("          [0]  Wether to be verbose");\
 sep_add_doc_line("");\
 sep_add_doc_line("CATEGORY");\
 sep_add_doc_line("        <util/cube>");\
 sep_add_doc_line("");

int main(int argc, char **argv){
sep3d input,output;
float *buf;
int   nh,i3,ndo,verb,max_size,n;
int   sect,nblock,axis,master;
int *nwind,*fwind,*jwind,*nsect,*fsect,*jsect;
int *nin,*fin,*jin,*nout,*fout,*jout;
int *ncopy, *fcopy,*jcopy;
char   pattern[256];
char *buf_in,*buf_out;
int esize,ns[2]; 
int master_data,i;
int assign[9999],nassign;
int write_head,write_grid;
int isect;
int copy,impi,nmpi,nloop;



#ifdef SEP_MPI
  MPI_Init(&argc,&argv);
   initpar(argc,argv);
  MPI_Comm_size(MPI_COMM_WORLD, &nmpi);
  MPI_Comm_rank(MPI_COMM_WORLD, &impi);
  if (impi==0) {
    if(0==getch("verb","d",&verb)) verb=0;
    mpi_sep_send_args(nmpi,10,verb);
  }
  else mpi_sep_receive_args();
  master_data=0;
  sect=nmpi;
  if(master_data==0) sect=sect-1;
#else
   initpar(argc,argv);
  master_data=1;
  sect=2;
  if(0==getch("verb","d",&verb)) verb=0;
#endif
  init_3d();
  MY_SEP_DOC
  doc(SOURCE);


/*get parameters*/
                                                                                                                     
if(0== getch("pattern","s",pattern)) strcpy(pattern,"BLOCK");
if(0==getch("max_size","d",&max_size)) max_size=50;

                                                                                                                     

/*initialize input*/
sep3d_initialize(&input);
if(0!= init_sep3d_tag("intag",&input,"INPUT"))
 seperr("trouble initializing input \n");

if(input.file_format[0]=='G') {write_head=1; write_grid=1;}
else if(input.file_format[0]=='H') {write_head=1; write_grid=0;}
else { write_head=0; write_grid=0;}



  getch_add_string("outtag.reassign=0");
  if(0!= init_sep3d_tag("outtag",&output,"SCRATCH"))
   seperr("trouble initializing input \n");

  if(output.nsect < 2)
    seperr("data not sectioned \n");

  if(0!=sep3dc_grab_sect_axis(&output,&axis))
    seperr("trouble grabbing sectioned axis \n");

  if(axis <= input.ndims) copy=0;
  else copy=1;

  nassign=getch("assign","d",assign);
  sect=output.nsect;
  if(nassign!=n-1) {
     fprintf(stderr, "assign length=%d nmpi=%d \n",nassign,n);
     seperr("only works in MPI mode number of assigns= nmpi-1\n");
  }
  for(i=0; i < nassign; i++){
     if(0!=sep3dc_assign_sect(&output,i+1,assign[i]))
      seperr("trouble assigning section \n");
  } 
  sep3dc_write_description("outtag",&output);



  if(input.data_format[0]=='C') esize=8;
  else if(input.data_format[0]=='B') esize=1;
  else esize=4;

  max_size=max_size*1000*1000/esize;
  init_loop_calc(input.ndims,input.n,"MAIN",max_size);

  nwind=(int*) malloc(sizeof(int)*input.ndims);
  fwind=(int*) malloc(sizeof(int)*input.ndims);
  jwind=(int*) malloc(sizeof(int)*input.ndims);

  if(copy==0){ /*data is split */
    jsect=(int*) malloc(sizeof(int)*input.ndims);
    nsect=(int*) malloc(sizeof(int)*input.ndims);
    fsect=(int*) malloc(sizeof(int)*input.ndims);
    jin=jsect; nin=nsect; fin=fsect;
    fout=fwind; jout=jwind; nout=nwind;
    nloop=nassign;
  }
  else{ /*mirroring data*/
    jcopy=(int*) malloc(sizeof(int)*output.ndims);
    ncopy=(int*) malloc(sizeof(int)*output.ndims);
    fcopy=(int*) malloc(sizeof(int)*output.ndims);
    jin=jwind; nin=nwind; fin=fwind;
    fout=fcopy; jout=jcopy; nout=ncopy;
    for(i=0; i < output.ndims; i++) jout[i]=1;
    for(i=0; i < output.ndims; i++) nout[i]=1;
    for(i=0; i < output.ndims; i++) fout[i]=0;
    fout[axis-1]= assign[ impi-1]-1;
    nloop=1;
     
  }
  for(i=0; i < input.ndims; i++) jwind[i]=1;


  while(0==do_sep_loop("MAIN",nwind,fwind)){

   for(isect=0;isect< nloop; isect++){

     if(copy==1){
       if(0!=sep3dc_section_window(&output,assign[isect]-1,nwind,fwind,
        jwind,nsect,fsect,jsect))
         seperr("trouble obtaining section window \n");
     } 
     else{
       for(i=0; i < input.ndims; i++){ nout[i]=nin[i]; fout[i]=fin[i];}
     }
     if(sep_thread_num()==0){
       if(verb>0){
          fprintf(stderr,"WINDOW section=%d  ",assign[isect]);
          for(i=0; i < input.ndims ; i++)
            fprintf(stderr,"f[%d]=%d n[%d]=%d ",i+1,fwind[i],i+1,nwind[i]);
          fprintf(stderr,"\n");
        }

        if(0!= sep3dc_grab_headers("intag",&input,&nh,&nin[1],&fin[1],&jin[1]))
          seperr("trouble grabbing headers \n");

        buf_in=(char*) malloc(esize*nh*input.n[0]);
                                                                                                                     
        if(0!=sep3dc_read_data("intag",&input,(void*)buf_in,nsect[0],
         fsect[0],jsect[0])) seperr("trouble reading data");
     }
     if(copy==0){ 
       if(0!= sep3dc_distribute_headers(&input,&output)) 
         seperr("trouble distributing headers");
     }
     else{
       if(0!= sep3dc_broadcast_headers(&input,0)) 
         seperr("trouble distributing headers");
       if(0!= sep3dc_header_copy(&input,&output)) 
         seperr("trouble copying headers");
     }
     if(0!=sep3dc_local_buffer_size(&output,ns,nout,fout,jout))
          seperr("trouble grabbing local buffer size \n");

     buf_out=(char*) malloc(esize*ns[0]*ns[1]);

     if(copy==0){
       if(0!=sep3dc_distribute_data(&input,&output,buf_in,buf_out)) 
          seperr("trouble distributing headers\n");
     }
     else{
       memcpy((void *)buf_out,(const void*) buf_in, esize*ns[0]*ns[1]);
       if(0!=sep3dc_broadcast_data(&input,ns[1],0,buf_out) )
          seperr("trouble broadsting data \n");
     } 
     if(0!= sep3dc_write_data("outtag",&output,(char*)buf_out,nout,fout,
      jout,nh, write_head,write_grid)) seperr("trouble writing outing data");
     free(buf_out); 

     if(sep_thread_num()==0) free(buf_in);

   } 
 }
/*   sep3dc_update_ntraces(&output);*/
/*   sep3d_rite_num_traces("outtag",&output); */

#ifdef SEP_MPI
  MPI_Finalize();
#endif

  return(SUCCESS);
}
