
/*
 * Copyright 1987 the Board of Trustees of the Leland Stanford Junior
 * University. Official permission to use this software is included in
 * the documentation. It authorizes you to use this file for any
 * non-commercial purpose, provided that this copyright notice is not
 * removed and that any modifications made to this file are commented
 * and dated in the style of my eXample below.
 */

/*
 *
 *  source file:   ./filters/xtlib/xtcommands.c
 *
 * Steve Cole (SEP), August 4 1991
 *      Inserted this sample edit history entry.
 *
 * Dave Nichols (SEP), February 14 1992
 *      Changed to new control structure, added new buttons
 *      for run, stop, prev.
 * Dave Nichols (SEP), April 30 1992
 *      Final version of the new xtpen, added message window
 * Dave Nichols (SEP), July 18 1992
 *      Added boxy mode for interact, it pops up a dialog to enter
 *	information for the label and it outputs a format suitable
 *	for the command line of Box
 */


#include "xtpen.h"

#include <X11/Xaw/Command.h>
#include <X11/Xaw/Label.h>

#include <X11/Xaw/Cardinals.h>
#include <X11/Shell.h>

/* variable that control the existance of buttons and labels */
static int wantButtons = 0;
static int  first_time = YES;
extern char     interact[];
static int wantLabels = 0;
static FILE *outfp;


/* The buttons that go into the control panel */
Widget quit_button, next_button, prev_button,  restart_button;
Widget run_button, stop_button, stretch_button;
Widget mode_button, frame_label, delay_label;


/* static variables that are set when a button is pressed */
static int didNEXT = NO, didPREV = NO, didQUIT = NO, didTIME=NO;
static int didFRAM1 = NO,  didRUN = NO, didSTOP=NO , didCHANGE = NO;


/* Function active? flags. We can't use XtIsSensitive because we might
 * not have active buttons */
static int next_on=NO ,prev_on=NO ,quit_on=NO ,restart_on=NO ;
static int run_on=NO , stop_on=NO, size_on=NO;


void  dummy_proc () { return; }
void actionDummy( w, ev, p, np)
Widget w; XEvent *ev; String *p; Cardinal *np; { dummy_proc(); }

void  next_proc(){  if( next_on == YES ) didNEXT = YES;    return; }
void actionNext( w, ev, p, np)
Widget w; XEvent *ev; String *p; Cardinal *np; { next_proc(); }

void  prev_proc(){  if( prev_on==YES ) didPREV = YES;    return; }
void actionPrev( w, ev, p, np)
Widget w; XEvent *ev; String *p; Cardinal *np; { prev_proc(); }

void quit_proc(){  if( quit_on == YES ) didQUIT = YES;    return; }

void actionQuit( w, ev, p, np)
Widget w; XEvent *ev; String *p; Cardinal *np; 
{
 if( interact[0] != '\0' ) {
if (first_time == YES){
        outfp = fopen (interact, "w");
        if (outfp == NULL) { 
            ERR (FATAL, name, "Can't open interact output file %s!", interact);
        }
        first_time = NO;
   }
/*	 fprintf( outfp,"\n");*/
  fflush( outfp );
}
  quit_proc(); 

}

void restart_proc(){  if( restart_on == YES) didFRAM1 = YES;    return; }
void actionRestart( w, ev, p, np)
Widget w; XEvent *ev; String *p; Cardinal *np; { restart_proc(); }

void run_proc(){  if( run_on == YES ) didRUN = YES;    return; }
void actionRun( w, ev, p, np)
Widget w; XEvent *ev; String *p; Cardinal *np; { run_proc(); }

void stop_proc(){  if( stop_on == YES ) didSTOP = YES;    return; }
void actionStop( w, ev, p, np)
Widget w; XEvent *ev; String *p; Cardinal *np; { stop_proc(); }

static Arg rigidArgs[] = { { XtNlabel, (XtArgVal)"Rigid" } };
static Arg stretchArgs[] = { { XtNlabel, (XtArgVal)"Stretchy" } };

void stretch_proc(){ 
    xt_stretchy = !xt_stretchy;  /* toggle the "stretchy" attribute */

    if( wantButtons ){

	if( xt_stretchy ){
	    /* if we are now stretchy change the label to "rigid" */
	    XtSetValues( stretch_button, rigidArgs, XtNumber(rigidArgs) );
	}else{
	    /* if we are now rigid change the label to "stretchy" */  
	    XtSetValues( stretch_button, stretchArgs, XtNumber(rigidArgs) );
	}

    }
    didCHANGE = YES; return; 
}
void actionStretch( w, ev, p, np)
Widget w; XEvent *ev; String *p; Cardinal *np; { stretch_proc(); }

/* the timeout proc called in run mode by an application timer */
void timeout_proc(){    didTIME = YES; return; }

void set_delay_label(delay)
float delay;
{ 
   Arg arg[1];
   char text[32];

   if( wantLabels ){
      sprintf( text, "delay %4.2f ", delay );
      XtSetArg( arg[0], XtNlabel, text );
      XtSetValues( delay_label , arg, ONE );
   }
}

/* procs to set pause between frames, these are not currently set as
 * button callbacks but they may be used in a translation table 
 */
void slower_proc(){ 
   if( fpause==0. ) fpause =.25; 
   else fpause = fpause*2.; 
   if( fpause >=10. ) fpause =9.99;

   set_delay_label(fpause);
}
void actionSlower( w, ev, p, np)
Widget w; XEvent *ev; String *p; Cardinal *np; { slower_proc(); }

void faster_proc(){ 
   fpause = fpause/2.; 
   if( fpause < .05 ) fpause =0.05;
   set_delay_label(fpause);
}
void actionFaster( w, ev, p, np)
Widget w; XEvent *ev; String *p; Cardinal *np; { faster_proc(); }


/* the number that is updated by user keystrokes */
static int inputNumber=0;

void actionNumber( w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{

    if( params && *params){
        int digits = strlen(*params);
        int value = atoi(*params);
        if( inputNumber == -1 ) inputNumber =0;
        while( digits > 0 ){
	    inputNumber *= 10;
	    digits--;
        }
        inputNumber += value;
    }
}

/* this is the global variable that is examained to see what the user typed */
int xt_next_num = -1;

void numberReset(){
    inputNumber = -1;
}
void actionNumReset( w, ev, p, np)
Widget w; XEvent *ev; String *p; Cardinal *np; { numberReset(); }


/* set next frame number to the users number and then do a next frame */
void gotoFrame(){
    xt_next_num = inputNumber ;
    if( next_on == YES ) didNEXT = YES;    return;
}
void actionGoto( w, ev, p, np)
Widget w; XEvent *ev; String *p; Cardinal *np; { gotoFrame(); }


/* toggle run mode between Forward, Backward, Both-ways */

/* this routine sets the mode label */
void set_mode_label(newlab)
char* newlab;
{ 
   Arg arg[1];
   char text[32];

   if( wantLabels ){
      strncpy( text, newlab, 32 );
      XtSetArg( arg[0], XtNlabel, text );
      XtSetValues( mode_button , arg, ONE );
   }
}
/* this is the global variable that controls run mode */
/* the enumeration is defined in xtpen.h */
int xt_run_mode=XT_FORWARD;

void toggle_run_mode(){
  switch( xt_run_mode ) {
    case XT_FORWARD:
        xt_run_mode = XT_BACKWARD;
	set_mode_label("Backwards");
	break;
    case XT_BACKWARD:
        xt_run_mode = XT_BOTH_WAYS;
	set_mode_label("Both Ways");
	break;
    case XT_BOTH_WAYS:
        xt_run_mode = XT_FORWARD;
	set_mode_label("Forwards ");
	break;
  }
	    
}
void actionRunMode( w, ev, p, np)
Widget w; XEvent *ev; String *p; Cardinal *np; { toggle_run_mode(); }


void set_frame_label(frame_num)
int frame_num;
{ 
   Arg arg[1];
   char text[10];

   if( wantLabels ){
       sprintf( text, " %3d ", frame_num );
       XtSetArg( arg[0], XtNlabel, text );
       XtSetValues( frame_label , arg, ONE );
   }
}

/* this action procedure prints the event location to the file "interact" */
void actionCoord(w,event,params,numparams)
Widget w;
XEvent *event;
String *params;
Cardinal *numparams;
{
int x,y;

   if( interact[0] == '\0' ) return;

   if (first_time == YES){
        outfp = fopen (interact, "w");
        if (outfp == NULL) {
            ERR (FATAL, name, "Can't open interact output file %s!", interact);
        }
        first_time = NO;
   }

   if( event->type == ButtonPress ){
       x = XCORD (event->xbutton.x);
       y = YCORD (event->xbutton.y);
       devtovpxy ( x, y, &x, &y );
       if( boxy ){
           doPointPopup( outfp, (float) x/RPERIN, (float) y/RPERIN );
       }else{
           fprintf( outfp,"%f\t%f\n",(float) x/RPERIN,(float) y/RPERIN );
           fflush( outfp );
       }

   }
}

void create_panel(parent,want_buttons,want_labels)
Widget parent;
int want_buttons, want_labels;
{
    wantButtons = want_buttons;
    wantLabels = want_labels;

    if( wantButtons ){
  
        next_button = XtVaCreateManagedWidget("Next",commandWidgetClass,
		parent, NULL);
        XtAddCallback(next_button, XtNcallback, next_proc, NULL);

        prev_button = XtVaCreateManagedWidget("Prev",commandWidgetClass,
		parent,NULL);
        XtAddCallback(prev_button, XtNcallback, prev_proc, NULL);

        quit_button = XtVaCreateManagedWidget("Quit",commandWidgetClass,
		parent,NULL);
        XtAddCallback(quit_button, XtNcallback, quit_proc, NULL);

        restart_button = XtVaCreateManagedWidget("Restart",commandWidgetClass,
		parent,NULL);
        XtAddCallback(restart_button, XtNcallback, restart_proc, NULL);

        run_button = XtVaCreateManagedWidget("Run",commandWidgetClass,
		parent,NULL);
        XtAddCallback(run_button, XtNcallback, run_proc, NULL);
 
        stop_button = XtVaCreateManagedWidget("Stop",commandWidgetClass,
		parent,NULL);
        XtAddCallback(stop_button, XtNcallback, stop_proc, NULL);   

        stretch_button = XtVaCreateManagedWidget("Stretchy",commandWidgetClass,
		parent,NULL);
        XtAddCallback(stretch_button, XtNcallback, stretch_proc, NULL);   
 	if( xt_stretchy )
	    XtSetValues( stretch_button, rigidArgs, XtNumber(rigidArgs) );

        mode_button = XtVaCreateManagedWidget("Forwards ",commandWidgetClass,
		parent,NULL);
        XtAddCallback(mode_button, XtNcallback, toggle_run_mode, NULL);   

    }

    if( wantLabels ){
        frame_label = XtCreateManagedWidget("  0 ",labelWidgetClass,parent,
			NULL,ZERO);

        delay_label = XtCreateManagedWidget("delay      ",
		labelWidgetClass,parent, NULL,ZERO);

    }

    return;
}
 
void activate_buttons()
{
    if( wantButtons ){
    /* turn on the buttons, if required */
    XtSetSensitive( next_button, ( next_on == YES ) );
    XtSetSensitive( prev_button, ( prev_on == YES ) );
    XtSetSensitive( quit_button, ( quit_on == YES ) );
    XtSetSensitive( restart_button, ( restart_on == YES ) );
    XtSetSensitive( run_button,  ( run_on == YES ) );
    XtSetSensitive( stop_button, ( stop_on == YES ) );
    XtSetSensitive( stretch_button, ( size_on == YES ) );
    }
 
return;
}

void inactivate_buttons()
{    
    /* turn all the buttons off */

    if( wantButtons ){
    XtSetSensitive( next_button, False );
    XtSetSensitive( prev_button, False );
    XtSetSensitive( quit_button, False );
    XtSetSensitive( restart_button, False );
    XtSetSensitive( run_button,  False );
    XtSetSensitive( stop_button, False );
    XtSetSensitive( stretch_button, False );
    }
   
    return;

}

int xt_pause( doNEXT,doPREV,doREST,doQUIT,doRUN,doSTOP,doXSIZE)
int doNEXT,doPREV,doREST,doQUIT,doRUN,doSTOP,doXSIZE;
{

    int retval;
    XtIntervalId id;


    next_on=doNEXT; prev_on=doPREV; quit_on=doQUIT; restart_on=doREST;
    run_on=doRUN;stop_on=doSTOP; size_on=doXSIZE;

    if( wantButtons ){
        activate_buttons();
    }

    if( doSTOP == YES ){ 
	unsigned long mpause; /* pause time in milliseconds */
	/* add a timeout event , add two millisecs to prevent zero */
        mpause = fpause*1000. + 2;
	didTIME = NO;
        id = XtAppAddTimeOut( pen_context,mpause, timeout_proc, NULL );

	/* we are in a run loop process events until the timer expires */
	/* only the stop and quit buttons will be active */

	while( !didTIME && !didSTOP && !didQUIT && !didNEXT && !didPREV ){
	    XtAppProcessEvent(pen_context,XtIMXEvent|XtIMTimer);
	}
	XtRemoveTimeOut(id);

	didTIME = NO;
	if( didSTOP ){
	    didSTOP=NO;	    return STOP;
        } else if (didQUIT) {
	       return  QUIT; 
	}else{
	    return TIME;
	}

    }else{
	/* if not in a run loop wait for a user event */
	while (!didNEXT && !didPREV && !didQUIT && 
	       !didFRAM1 && !didRUN && !didSTOP &&!didCHANGE) {
	    XtAppProcessEvent(pen_context,XtIMXEvent);
	}
  
	if (didNEXT) {
	        didNEXT = NO;	retval = NEXT;
        } else if (didPREV) {
		didPREV = NO;    retval = PREV; 
        } else if (didRUN) {
		didRUN = NO;     retval= RUN; 
        } else if (didSTOP) {
		didSTOP = NO;    retval = STOP; 
        } else if (didQUIT) {
	       retval = QUIT; 
        } else if (didFRAM1) {
		didFRAM1 = NO;	 retval = RESTART;  
        } else if (didCHANGE) {
		didCHANGE = NO;	 retval = CHANGED;  
        }
	if( wantButtons ){
	   inactivate_buttons();
        }
	return retval;
    }

}

